from ibm_watsonx_data_integration.services.datastage.models.flow.datastage_flow import DataStageFlow
from ibm_watsonx_data_integration.services.datastage.tests.integration.util import AbstractFlowRunTest
from os import environ
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestAzureDatabricksConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "azuredatabricks_peek"

    def get_fc(self, project: Project):
        fc = DataStageFlow(project=project, name=self.get_flow_name(), description="", flow_type="datastage")

        # Stages
        _yyyymmdd_1_ = fc.add_stage("Microsoft Azure Databricks", "yyyymmdd_1")
        _yyyymmdd_1_.configuration.catalog_name = "test"
        _yyyymmdd_1_.configuration.schema_name = "default"
        _yyyymmdd_1_.configuration.table_name = "yyyy-mm-dd"

        _yyyymmdd_1_.configuration.connection.authentication_method = "entra_id"
        _yyyymmdd_1_.configuration.connection.microsoft_entra_id_token = environ["AZUREDATABRICKS_ENTRA_ID_TOKEN"]
        _yyyymmdd_1_.configuration.connection.hostname_or_ip_address = environ["AZUREDATABRICKS_HOST"]
        _yyyymmdd_1_.configuration.connection.http_path = "/sql/1.0/warehouses/d09152f0d320e3cc"
        _yyyymmdd_1_.configuration.connection.port = "443"

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        yyyymmdd_1_schema = _yyyymmdd_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        yyyymmdd_1_schema.add_field("VARCHAR", "date1").nullable().length(255)
        yyyymmdd_1_schema.add_field("VARCHAR", "date2").nullable().length(255)
        yyyymmdd_1_schema.add_field("BIGINT", "id").nullable()

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import AMAZONS3_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class AuthMethod(Enum):
    basic_credentials = "basic_credentials"
    temporary_credentials = "temporary_credentials"
    trusted_role_credentials = "trusted_role_credentials"


class Amazons3Conn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "a0b1d14a-4767-404c-aac1-4ce0e62818c3"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    access_key: str | None = Field(None, alias="access_key")
    bucket: str | None = Field(None, alias="bucket")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    proxy: bool | None = Field(False, alias="proxy")
    proxy_host: str = Field(None, alias="proxy_host")
    proxy_password: str | None = Field(None, alias="proxy_password")
    proxy_port: int = Field(None, alias="proxy_port")
    proxy_username: str | None = Field(None, alias="proxy_user")
    region: str | None = Field(None, alias="region")
    secret_key: str = Field(None, alias="secret_key")
    url: str | None = Field(None, alias="url")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    ds_host: str | None = Field(None, alias="_host")
    ds_port: str | None = Field(None, alias="_port")
    authentication_method: AMAZONS3_CONNECTION.AuthMethod | None = Field(None, alias="auth_method")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    duration_seconds: int | None = Field(None, alias="duration_seconds")
    external_id: str | None = Field(None, alias="external_id")
    role_arn: str | None = Field(None, alias="role_arn")
    role_session_name: str = Field(None, alias="role_session_name")
    session_token: str | None = Field(None, alias="session_token")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        include.add("proxy_port") if (self.proxy) else exclude.add("proxy_port")
        (
            include.add("secret_key")
            if (
                (((not self.defer_credentials)) and ((self.access_key)))
                or (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "basic_credentials"
                                )
                                or (self.authentication_method == "basic_credentials")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "temporary_credentials"
                            )
                            or (self.authentication_method == "temporary_credentials")
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "trusted_role_credentials"
                            )
                            or (self.authentication_method == "trusted_role_credentials")
                        )
                    )
                )
            )
            else exclude.add("secret_key")
        )
        include.add("proxy_host") if (self.proxy) else exclude.add("proxy_host")
        (
            include.add("access_key")
            if (
                ((not self.defer_credentials))
                and (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "basic_credentials"
                                )
                                or (self.authentication_method == "basic_credentials")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "temporary_credentials"
                            )
                            or (self.authentication_method == "temporary_credentials")
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "trusted_role_credentials"
                            )
                            or (self.authentication_method == "trusted_role_credentials")
                        )
                    )
                )
            )
            else exclude.add("access_key")
        )
        include.add("proxy_username") if (self.proxy) else exclude.add("proxy_username")
        include.add("proxy_password") if (self.proxy) else exclude.add("proxy_password")
        (
            include.add("session_token")
            if (
                (
                    ((not self.defer_credentials))
                    and ((self.access_key))
                    and ((not self.role_arn))
                    and (
                        (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "temporary_credentials"
                                    )
                                    or (self.authentication_method == "temporary_credentials")
                                )
                            )
                        )
                    )
                )
                or (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "temporary_credentials"
                        )
                        or (self.authentication_method == "temporary_credentials")
                    )
                )
            )
            else exclude.add("session_token")
        )
        (
            include.add("duration_seconds")
            if (
                (
                    ((not self.defer_credentials))
                    and ((self.role_arn))
                    and (
                        (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "trusted_role_credentials"
                                    )
                                    or (self.authentication_method == "trusted_role_credentials")
                                )
                            )
                        )
                    )
                )
                or (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "trusted_role_credentials"
                        )
                        or (self.authentication_method == "trusted_role_credentials")
                    )
                )
            )
            else exclude.add("duration_seconds")
        )
        (
            include.add("role_arn")
            if (
                (
                    ((not self.defer_credentials))
                    and ((self.access_key))
                    and ((not self.session_token))
                    and (
                        (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "trusted_role_credentials"
                                    )
                                    or (self.authentication_method == "trusted_role_credentials")
                                )
                            )
                        )
                    )
                )
                or (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "trusted_role_credentials"
                        )
                        or (self.authentication_method == "trusted_role_credentials")
                    )
                )
            )
            else exclude.add("role_arn")
        )
        (
            include.add("role_session_name")
            if (
                (
                    ((not self.defer_credentials))
                    and ((self.role_arn))
                    and (
                        (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "trusted_role_credentials"
                                    )
                                    or (self.authentication_method == "trusted_role_credentials")
                                )
                            )
                        )
                    )
                )
                or (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "trusted_role_credentials"
                        )
                        or (self.authentication_method == "trusted_role_credentials")
                    )
                )
            )
            else exclude.add("role_session_name")
        )
        (
            include.add("external_id")
            if (
                (
                    ((not self.defer_credentials))
                    and (
                        (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "trusted_role_credentials"
                                    )
                                    or (self.authentication_method == "trusted_role_credentials")
                                )
                            )
                        )
                    )
                )
                or (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "trusted_role_credentials"
                        )
                        or (self.authentication_method == "trusted_role_credentials")
                    )
                )
            )
            else exclude.add("external_id")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")

        (
            include.add("role_arn")
            if (
                (not self.defer_credentials)
                and (self.access_key)
                and (not self.session_token)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "trusted_role_credentials" in str(self.authentication_method.value)
                        )
                        or ("trusted_role_credentials" in str(self.authentication_method))
                    )
                )
            )
            or (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "trusted_role_credentials" in str(self.authentication_method.value)
                    )
                    or ("trusted_role_credentials" in str(self.authentication_method))
                )
            )
            else exclude.add("role_arn")
        )
        (
            include.add("duration_seconds")
            if (
                (not self.defer_credentials)
                and (self.role_arn)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "trusted_role_credentials" in str(self.authentication_method.value)
                        )
                        or ("trusted_role_credentials" in str(self.authentication_method))
                    )
                )
            )
            or (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "trusted_role_credentials" in str(self.authentication_method.value)
                    )
                    or ("trusted_role_credentials" in str(self.authentication_method))
                )
            )
            else exclude.add("duration_seconds")
        )
        include.add("proxy_username") if (self.proxy) else exclude.add("proxy_username")
        (
            include.add("secret_key")
            if ((not self.defer_credentials) and (self.access_key))
            or (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "basic_credentials" in str(self.authentication_method.value)
                    )
                    or ("basic_credentials" in str(self.authentication_method))
                )
                or self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "temporary_credentials" in str(self.authentication_method.value)
                    )
                    or ("temporary_credentials" in str(self.authentication_method))
                )
                or self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "trusted_role_credentials" in str(self.authentication_method.value)
                    )
                    or ("trusted_role_credentials" in str(self.authentication_method))
                )
            )
            else exclude.add("secret_key")
        )
        include.add("proxy_port") if (self.proxy) else exclude.add("proxy_port")
        (
            include.add("role_session_name")
            if (
                (not self.defer_credentials)
                and (self.role_arn)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "trusted_role_credentials" in str(self.authentication_method.value)
                        )
                        or ("trusted_role_credentials" in str(self.authentication_method))
                    )
                )
            )
            or (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "trusted_role_credentials" in str(self.authentication_method.value)
                    )
                    or ("trusted_role_credentials" in str(self.authentication_method))
                )
            )
            else exclude.add("role_session_name")
        )
        (
            include.add("session_token")
            if (
                (not self.defer_credentials)
                and (self.access_key)
                and (not self.role_arn)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "temporary_credentials" in str(self.authentication_method.value)
                        )
                        or ("temporary_credentials" in str(self.authentication_method))
                    )
                )
            )
            or (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "temporary_credentials" in str(self.authentication_method.value)
                    )
                    or ("temporary_credentials" in str(self.authentication_method))
                )
            )
            else exclude.add("session_token")
        )
        (
            include.add("external_id")
            if (
                (not self.defer_credentials)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "trusted_role_credentials" in str(self.authentication_method.value)
                        )
                        or ("trusted_role_credentials" in str(self.authentication_method))
                    )
                )
            )
            or (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "trusted_role_credentials" in str(self.authentication_method.value)
                    )
                    or ("trusted_role_credentials" in str(self.authentication_method))
                )
            )
            else exclude.add("external_id")
        )
        include.add("proxy_password") if (self.proxy) else exclude.add("proxy_password")
        (
            include.add("access_key")
            if (not self.defer_credentials)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "basic_credentials" in str(self.authentication_method.value)
                    )
                    or ("basic_credentials" in str(self.authentication_method))
                )
                and self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "temporary_credentials" in str(self.authentication_method.value)
                    )
                    or ("temporary_credentials" in str(self.authentication_method))
                )
                and self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "trusted_role_credentials" in str(self.authentication_method.value)
                    )
                    or ("trusted_role_credentials" in str(self.authentication_method))
                )
            )
            else exclude.add("access_key")
        )
        include.add("proxy_host") if (self.proxy) else exclude.add("proxy_host")
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



from ibm_watsonx_data_integration.services.datastage.models.flow.dag import (
    DAG,
    Link,
    Node,
)
from ibm_watsonx_data_integration.services.datastage.models.stages import *


class StageComposer:
    def __init__(self):
        self.dag = DAG()

    def get_dag(self):
        return self.dag

    def get_link(self, left: Node, right: Node) -> Link | None:
        links = self.dag.get_links_between(left, right)
        if len(links) == 0:
            raise ValueError("No link between nodes")
        if len(links) > 1:
            raise ValueError("Multiple links between nodes")
        return links[0]

    def get_links(self, left: Node, right: Node) -> list[Link]:
        return self.dag.get_links_between(left, right)

    def add(self, node: Node):
        self.dag.add_node(node)

    def remove(self, node: Node):
        self.dag.remove_node(node)

    def add_link(self, src: Node, dest: Node, link: Link = None) -> Link:
        if not link:
            link = Link(self, name=None)
        link.src = src
        link.dest = dest
        self.dag.add_link(link)
        return link

    def remove_links(self, src: Node, dest: Node) -> list[Link]:
        removed = self.dag.remove_links(src, dest)
        if not removed:
            raise ValueError(f"No links found between {src} and {dest}")
        return removed

    def remove_link(self, src: Node, dest: Node, link_name: str) -> Link:
        removed = self.dag.remove_link(src, dest, link_name)
        if not removed:
            raise ValueError(f"Link {link_name} not found between {src} and {dest}")
        if len(removed) > 1:
            # console.print(
            #     f"WARN: Multiple links found with name {link_name}. Returning first one"
            # )
            print(f"WARN: Multiple links found with name {link_name}. Returning first one")
        return removed[0]



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import ORACLE_DATASTAGE_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class ConnectionType(Enum):
    ldap = "ldap"
    seps = "seps"
    tcp = "tcp"
    tcps = "tcps"


class OracleDatastageConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "dd22f798-8c9b-41fa-841e-d66cbdf50722"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    cas_lite_service_authorization_header: str | None = Field(None, alias="cas_lite_auth_header")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    connection_string: str = Field(None, alias="connection_string")
    connection_type: ORACLE_DATASTAGE_CONNECTION.ConnectionType | None = Field(
        ORACLE_DATASTAGE_CONNECTION.ConnectionType.tcp, alias="connection_type"
    )
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    gateway_url: str | None = Field(None, alias="gateway_url")
    alternate_servers: str | None = Field(None, alias="oracle_alternate_servers")
    hostname: str = Field(None, alias="oracle_db_host")
    port: str = Field(None, alias="oracle_db_port")
    servicename: str = Field(None, alias="oracle_service_name")
    password: str | None = Field(None, alias="password")
    rac_server_name: str | None = Field(None, alias="rac_name")
    satellite_client_certificate: str | None = Field(None, alias="sl_client_cert")
    satellite_client_private_key: str | None = Field(None, alias="sl_client_private_key")
    satellite_connector_id: str | None = Field(None, alias="sl_connector_id")
    satellite_endpoint_host: str | None = Field(None, alias="sl_endpoint_host")
    satellite_endpoint_display_name: str | None = Field(None, alias="sl_endpoint_name")
    satellite_endpoint_port: int | None = Field(None, alias="sl_endpoint_port")
    original_hostname_of_the_resource: str | None = Field(None, alias="sl_host_original")
    satellite_as_http_proxy: bool | None = Field(None, alias="sl_http_proxy")
    satellite_location_id: str | None = Field(None, alias="sl_location_id")
    satellite_service_url: str | None = Field(None, alias="sl_service_url")
    use_cas_lite_service: bool | None = Field(True, alias="use_cas_lite")
    use_connection_string: bool | None = Field(False, alias="use_connection_string")
    username: str | None = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    oracle_client_version: str | None = Field(None, alias="version")
    xa_database_name: str | None = Field(None, alias="xao_db_name")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        (
            include.add("alternate_servers")
            if (
                ((not self.use_connection_string))
                and (
                    (
                        (
                            (
                                self.connection_type
                                and (
                                    (hasattr(self.connection_type, "value") and self.connection_type.value == "tcp")
                                    or (self.connection_type == "tcp")
                                )
                            )
                        )
                        or (
                            (
                                self.connection_type
                                and (
                                    (hasattr(self.connection_type, "value") and self.connection_type.value == "tcps")
                                    or (self.connection_type == "tcps")
                                )
                            )
                        )
                    )
                    or (
                        self.connection_type
                        and (
                            (hasattr(self.connection_type, "value") and not self.connection_type.value)
                            or (not self.connection_type)
                        )
                    )
                )
            )
            else exclude.add("alternate_servers")
        )
        (
            include.add("password")
            if (
                ((not self.defer_credentials))
                and (
                    ((self.use_connection_string))
                    or (
                        ((not self.use_connection_string))
                        and (
                            (
                                (
                                    (
                                        self.connection_type
                                        and (
                                            (
                                                hasattr(self.connection_type, "value")
                                                and self.connection_type.value == "ldap"
                                            )
                                            or (self.connection_type == "ldap")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.connection_type
                                        and (
                                            (
                                                hasattr(self.connection_type, "value")
                                                and self.connection_type.value == "tcp"
                                            )
                                            or (self.connection_type == "tcp")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.connection_type
                                        and (
                                            (
                                                hasattr(self.connection_type, "value")
                                                and self.connection_type.value == "tcps"
                                            )
                                            or (self.connection_type == "tcps")
                                        )
                                    )
                                )
                            )
                            or (
                                self.connection_type
                                and (
                                    (hasattr(self.connection_type, "value") and not self.connection_type.value)
                                    or (not self.connection_type)
                                )
                            )
                        )
                    )
                )
            )
            else exclude.add("password")
        )
        include.add("connection_type") if (not self.use_connection_string) else exclude.add("connection_type")
        include.add("servicename") if (not self.use_connection_string) else exclude.add("servicename")
        (
            include.add("use_cas_lite_service")
            if (self.cas_lite_service_authorization_header)
            else exclude.add("use_cas_lite_service")
        )
        (
            include.add("port")
            if (
                ((not self.use_connection_string))
                and (
                    (
                        (
                            (
                                self.connection_type
                                and (
                                    (hasattr(self.connection_type, "value") and self.connection_type.value == "tcp")
                                    or (self.connection_type == "tcp")
                                )
                            )
                        )
                        or (
                            (
                                self.connection_type
                                and (
                                    (hasattr(self.connection_type, "value") and self.connection_type.value == "tcps")
                                    or (self.connection_type == "tcps")
                                )
                            )
                        )
                    )
                    or (
                        self.connection_type
                        and (
                            (hasattr(self.connection_type, "value") and not self.connection_type.value)
                            or (not self.connection_type)
                        )
                    )
                )
            )
            else exclude.add("port")
        )
        (
            include.add("cas_lite_service_authorization_header")
            if (self.use_cas_lite_service)
            else exclude.add("cas_lite_service_authorization_header")
        )
        (
            include.add("hostname")
            if (
                ((not self.use_connection_string))
                and (
                    (
                        (
                            (
                                self.connection_type
                                and (
                                    (hasattr(self.connection_type, "value") and self.connection_type.value == "tcp")
                                    or (self.connection_type == "tcp")
                                )
                            )
                        )
                        or (
                            (
                                self.connection_type
                                and (
                                    (hasattr(self.connection_type, "value") and self.connection_type.value == "tcps")
                                    or (self.connection_type == "tcps")
                                )
                            )
                        )
                    )
                    or (
                        self.connection_type
                        and (
                            (hasattr(self.connection_type, "value") and not self.connection_type.value)
                            or (not self.connection_type)
                        )
                    )
                )
            )
            else exclude.add("hostname")
        )
        include.add("connection_string") if (self.use_connection_string) else exclude.add("connection_string")
        (
            include.add("username")
            if (
                ((not self.defer_credentials))
                and (
                    ((self.use_connection_string))
                    or (
                        ((not self.use_connection_string))
                        and (
                            (
                                (
                                    (
                                        self.connection_type
                                        and (
                                            (
                                                hasattr(self.connection_type, "value")
                                                and self.connection_type.value == "ldap"
                                            )
                                            or (self.connection_type == "ldap")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.connection_type
                                        and (
                                            (
                                                hasattr(self.connection_type, "value")
                                                and self.connection_type.value == "tcp"
                                            )
                                            or (self.connection_type == "tcp")
                                        )
                                    )
                                )
                                or (
                                    (
                                        self.connection_type
                                        and (
                                            (
                                                hasattr(self.connection_type, "value")
                                                and self.connection_type.value == "tcps"
                                            )
                                            or (self.connection_type == "tcps")
                                        )
                                    )
                                )
                            )
                            or (
                                self.connection_type
                                and (
                                    (hasattr(self.connection_type, "value") and not self.connection_type.value)
                                    or (not self.connection_type)
                                )
                            )
                        )
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("alternate_servers") if (self.hidden_dummy_property1) else exclude.add("alternate_servers")
        include.add("connection_type") if (self.hidden_dummy_property1) else exclude.add("connection_type")
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")

        (
            include.add("use_cas_lite_service")
            if (self.cas_lite_service_authorization_header)
            else exclude.add("use_cas_lite_service")
        )
        (
            include.add("hostname")
            if (self.use_connection_string == "false" or not self.use_connection_string)
            and (
                (
                    self.connection_type
                    and (
                        (
                            hasattr(self.connection_type, "value")
                            and self.connection_type.value
                            and "tcp" in str(self.connection_type.value)
                        )
                        or ("tcp" in str(self.connection_type))
                    )
                    or self.connection_type
                    and (
                        (
                            hasattr(self.connection_type, "value")
                            and self.connection_type.value
                            and "tcps" in str(self.connection_type.value)
                        )
                        or ("tcps" in str(self.connection_type))
                    )
                )
                or (
                    self.connection_type
                    and (
                        (hasattr(self.connection_type, "value") and not self.connection_type.value)
                        or (not self.connection_type)
                    )
                )
            )
            else exclude.add("hostname")
        )
        (
            include.add("cas_lite_service_authorization_header")
            if (self.use_cas_lite_service == "true" or self.use_cas_lite_service)
            else exclude.add("cas_lite_service_authorization_header")
        )
        (
            include.add("alternate_servers")
            if (self.use_connection_string == "false" or not self.use_connection_string)
            and (
                (
                    self.connection_type
                    and (
                        (
                            hasattr(self.connection_type, "value")
                            and self.connection_type.value
                            and "tcp" in str(self.connection_type.value)
                        )
                        or ("tcp" in str(self.connection_type))
                    )
                    or self.connection_type
                    and (
                        (
                            hasattr(self.connection_type, "value")
                            and self.connection_type.value
                            and "tcps" in str(self.connection_type.value)
                        )
                        or ("tcps" in str(self.connection_type))
                    )
                )
                or (
                    self.connection_type
                    and (
                        (hasattr(self.connection_type, "value") and not self.connection_type.value)
                        or (not self.connection_type)
                    )
                )
            )
            else exclude.add("alternate_servers")
        )
        (
            include.add("servicename")
            if (self.use_connection_string == "false" or not self.use_connection_string)
            else exclude.add("servicename")
        )
        (
            include.add("connection_string")
            if (self.use_connection_string == "true" or self.use_connection_string)
            else exclude.add("connection_string")
        )
        (
            include.add("password")
            if (not self.defer_credentials)
            and (
                (self.use_connection_string == "true" or self.use_connection_string)
                or (
                    (self.use_connection_string == "false" or not self.use_connection_string)
                    and (
                        (
                            self.connection_type
                            and (
                                (
                                    hasattr(self.connection_type, "value")
                                    and self.connection_type.value
                                    and "ldap" in str(self.connection_type.value)
                                )
                                or ("ldap" in str(self.connection_type))
                            )
                            or self.connection_type
                            and (
                                (
                                    hasattr(self.connection_type, "value")
                                    and self.connection_type.value
                                    and "tcp" in str(self.connection_type.value)
                                )
                                or ("tcp" in str(self.connection_type))
                            )
                            or self.connection_type
                            and (
                                (
                                    hasattr(self.connection_type, "value")
                                    and self.connection_type.value
                                    and "tcps" in str(self.connection_type.value)
                                )
                                or ("tcps" in str(self.connection_type))
                            )
                        )
                        or (
                            self.connection_type
                            and (
                                (hasattr(self.connection_type, "value") and not self.connection_type.value)
                                or (not self.connection_type)
                            )
                        )
                    )
                )
            )
            else exclude.add("password")
        )
        (
            include.add("connection_type")
            if (self.use_connection_string == "false" or not self.use_connection_string)
            else exclude.add("connection_type")
        )
        (
            include.add("username")
            if (not self.defer_credentials)
            and (
                (self.use_connection_string == "true" or self.use_connection_string)
                or (
                    (self.use_connection_string == "false" or not self.use_connection_string)
                    and (
                        (
                            self.connection_type
                            and (
                                (
                                    hasattr(self.connection_type, "value")
                                    and self.connection_type.value
                                    and "ldap" in str(self.connection_type.value)
                                )
                                or ("ldap" in str(self.connection_type))
                            )
                            or self.connection_type
                            and (
                                (
                                    hasattr(self.connection_type, "value")
                                    and self.connection_type.value
                                    and "tcp" in str(self.connection_type.value)
                                )
                                or ("tcp" in str(self.connection_type))
                            )
                            or self.connection_type
                            and (
                                (
                                    hasattr(self.connection_type, "value")
                                    and self.connection_type.value
                                    and "tcps" in str(self.connection_type.value)
                                )
                                or ("tcps" in str(self.connection_type))
                            )
                        )
                        or (
                            self.connection_type
                            and (
                                (hasattr(self.connection_type, "value") and not self.connection_type.value)
                                or (not self.connection_type)
                            )
                        )
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("port")
            if (self.use_connection_string == "false" or not self.use_connection_string)
            and (
                (
                    self.connection_type
                    and (
                        (
                            hasattr(self.connection_type, "value")
                            and self.connection_type.value
                            and "tcp" in str(self.connection_type.value)
                        )
                        or ("tcp" in str(self.connection_type))
                    )
                    or self.connection_type
                    and (
                        (
                            hasattr(self.connection_type, "value")
                            and self.connection_type.value
                            and "tcps" in str(self.connection_type.value)
                        )
                        or ("tcps" in str(self.connection_type))
                    )
                )
                or (
                    self.connection_type
                    and (
                        (hasattr(self.connection_type, "value") and not self.connection_type.value)
                        or (not self.connection_type)
                    )
                )
            )
            else exclude.add("port")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



import json
import os
from pathlib import Path
from typing import Literal
import traceback

import ibm_watsonx_data_integration.services.datastage.models.flow_json_model as models
from ibm_watsonx_data_integration.services.datastage.codegen.code_generator import (
    FlowCodeGenerator,
    MasterCodeGenerator,
    # TestCaseCodeGenerator,
)
from ibm_watsonx_data_integration.services.datastage.codegen.dag_generator import DAGGenerator
from ibm_watsonx_data_integration.services.datastage.codegen.exporters.flow_file_exporter import FlowFileExporter
from ibm_watsonx_data_integration.services.datastage.codegen.exporters.single_file_exporter import SingleFileExporter
from ibm_watsonx_data_integration.services.datastage.codegen.exporters.util import (
    _autogenerated_header,
)
from ibm_watsonx_data_integration.services.datastage.codegen.importers import ZipImporter

# from ibm_watsonx_data_integration.services.datastage.models.test_case_composer import TestCaseComposer


def _delete_output_path(output_path: str):
    """
    Deletes the output path if it exists.
    """
    if os.path.exists(output_path):
        if os.path.isfile(output_path):
            os.remove(output_path)
        elif os.path.isdir(output_path):
            for root, dirs, files in os.walk(output_path, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    os.rmdir(os.path.join(root, name))
            os.rmdir(output_path)


def _get_extension(file_path: str) -> str:
    """
    Returns the file extension of the given file path.
    """
    return Path(file_path).suffix.lstrip(".").lower()


ExportMode = Literal["single_file", "file_per_asset", "file_per_flow"]


# need to have options for:
# converting everything locally
# using api for conversion
# preserve positioning
# optimize
class PythonGeneratorConfig:
    """
    The underlying settngs for the python generator.

    Args:
        mode: The export mode, or how the generated code will be structured. Options are:
            - ``single_file``: All code in one file.
            - ``file_per_flow``: (default) Each flow in its own file, with relevant assets (connections, subflows, parameter sets, components, etc) included as needed.
        create_job: Whether to create a job for the flow(s) (default is True).
        run_job: Whether to run the created job
        overwrite: Whether to overwrite the directory being written to (default is False).
        use_flow_name: Whether the run/create flow statement should use the flow's original name (applies for zip structure only, default is True).
        api_key: TODO
        project_id: TODO
    """

    def __init__(
        self,
        *,
        mode: ExportMode = "file_per_flow",
        create_job: bool = True,
        run_job: bool = True,
        overwrite: bool = False,
        use_flow_name: bool = True,
        api_key: str = "<TODO: insert your api_key>",
        project_id: str = "<TODO: insert your project_id>",
    ):
        self.create_job = create_job
        self.run_job = run_job
        self.mode = mode
        self.overwrite = overwrite
        self.use_flow_name = use_flow_name
        self.api_key = api_key
        self.project_id = project_id


class PythonGenerator:
    """
    Converts DataStage flows exported as a flow JSON or ZIP file to Python SDK code.
    """

    def __init__(self):
        self.configuration = PythonGeneratorConfig()

    def get_code(self, input_path):
        """
        Returns the generated code as a string. Applicable only for JSON inputs.
        """
        if _get_extension(input_path) != "json":
            raise ValueError(f"Input file must be a JSON file, got '{input_path}'")

        with open(input_path, "r") as f:
            flow_json = json.load(f)

        try:
            flow_model = models.Flow(**flow_json)
        except Exception:
            try:
                flow_model = models.Flow(**flow_json["attachments"])
            except Exception:
                try:
                    flow_json = flow_json[0]
                    flow_model = models.Flow(**flow_json)
                except Exception as e:
                    raise ValueError(f"Invalid flow JSON: {e}")

        dag_gen = DAGGenerator(flow_model)
        fc = dag_gen.generate()
        code_gen = FlowCodeGenerator(fc=fc, master_gen=MasterCodeGenerator())
        header = _autogenerated_header()
        setup = "\n".join(code_gen.generate_setup(self.configuration.api_key, self.configuration.project_id))
        code = code_gen.generate_all()
        return header + setup + code, code_gen

    # def get_test_case_code(self, input_path):
    #     """
    #     Returns the generated code as a string. Applicable only for JSON inputs.
    #     """
    #     with open(input_path, "r") as f:
    #         flow_json = json.load(f)

    #     tcc = TestCaseComposer.from_dict(flow_json)
    #     code_gen = TestCaseCodeGenerator(test_case_composer=tcc, master_gen=MasterCodeGenerator())
    #     code = "from ibm.datastage._framework.test_case_composer import TestCaseComposer\n"
    #     code += "from ibm.datastage._framework.sdk import DataStageSDK\n"
    #     code += "from ibm.datastage.config.config import AutoDetectConfig\n\n"
    #     code += "config = AutoDetectConfig()\n"
    #     code += "sdk = DataStageSDK(config)\n"
    #     code += code_gen.generate_code()
    #     return code

    def _generate_from_json(self, input_path, output_path=None):
        generated_code = ""
        errors = {}

        try:
            generated_code, code_gen = self.get_code(input_path)
            generated_code += f"\n\nproject.update_flow({code_gen.composer})"
            if self.configuration.create_job:
                job_name = code_gen.flow_name + "_job"
                job_var = "job_1"
                generated_code += f'\n\n{job_var} = project.create_job(name="{job_name}", flow=flow)'
                if self.configuration.run_job:
                    job_run_var = "job_run_1"
                    generated_code += f'\n\n{job_run_var} = {job_var}.start(name="{job_name}_job", description="")'
            if output_path:
                with open(output_path, "w") as f:
                    f.write(generated_code)
        except Exception:
            errors[Path(input_path).stem] = traceback.format_exc()
        return {output_path: generated_code}, errors

    def _generate_from_zip(self, input_path, output_path=None):
        zip_importer = ZipImporter(input_path)
        zip_importer.run()

        match self.configuration.mode:
            case "single_file":
                return SingleFileExporter(
                    zip_importer=zip_importer,
                    output_path=output_path,
                    create_job=self.configuration.create_job,
                    run_job=self.configuration.run_job,
                    use_flow_name=self.configuration.use_flow_name,
                    api_key=self.configuration.api_key,
                    project_id=self.configuration.project_id,
                ).run()
            case "file_per_flow":
                return FlowFileExporter(
                    zip_importer=zip_importer,
                    output_path=output_path,
                    create_job=self.configuration.create_job,
                    run_job=self.configuration.run_job,
                    use_flow_name=self.configuration.use_flow_name,
                    api_key=self.configuration.api_key,
                    project_id=self.configuration.project_id,
                ).run()
            # case "file_per_asset": # Removing this feature for now
            #     return MultiFileExporter(
            #         zip_importer=zip_importer,
            #         output_path=output_path,
            #         create_job=self.configuration.create_job,
            #         use_flow_name=self.configuration.use_flow_name,
            #     ).run()

    def generate(self, input_path, output_path=None):
        """
        Runs the Python converter, generating Python code and writing it to the specified output path.

        Args:
            input_path: The path to the input file. Can be a flow JSON file or a ZIP file containing the flow and its dependencies.
            output_path (optional): The output path where the generated Python code will be written.

        Returns:
            A dictionary containing the generated code strings.
        """

        if not input_path:
            raise ValueError("Input path must be specified.")

        if self.configuration.overwrite:
            _delete_output_path(output_path)

        match _get_extension(input_path):
            case "json":
                output_path = output_path or "code"
                generated_code, errors = self._generate_from_json(input_path, output_path)
                for filename, err in errors.items():
                    print(err)
                return generated_code, errors
            case "zip":
                generated_code, errors = self._generate_from_zip(input_path, output_path)
                for filename, err in errors.items():
                    print(err)
                return generated_code, errors
            case _:
                raise ValueError(f"Unsupported input file type: '{input_path}'. Only .json and .zip files are supported.")



# IBM Confidential

"""This module contains the API for the DataStage CPD Service."""

from ibm_watsonx_data_integration.services.datastage.api.datastage_flow_api import DatastageFlowApiClient

__all__ = [
    "DatastageFlowApiClient",
]



import warnings
from ibm_watsonx_data_integration.services.datastage.models.base_stage import BaseStage
from ibm_watsonx_data_integration.services.datastage.models.connections.apache_kafka_connection import ApacheKafkaConn
from ibm_watsonx_data_integration.services.datastage.models.enums import APACHE_KAFKA
from pydantic import Field
from typing import ClassVar


class apache_kafka(BaseStage):

    op_name: ClassVar[str] = "KafkaConnectorPX"
    node_type: ClassVar[str] = "binding"
    image: ClassVar[str] = "/data-intg/flows/graphics/palette/KafkaConnectorPX.svg"
    label: ClassVar[str] = "Apache Kafka"
    runtime_column_propagation: bool = Field(False, alias="runtime_column_propagation")
    connection: ApacheKafkaConn = ApacheKafkaConn()
    advanced_kafka_config_options: str | None = Field(None, alias="advanced_kafka_config_options")
    buf_free_run_ronly: int | None = Field(50, alias="buf_free_run_ronly")
    buf_mode_ronly: APACHE_KAFKA.BufModeRonly | None = Field(
        APACHE_KAFKA.BufModeRonly.default, alias="buf_mode_ronly"
    )
    buffer_free_run_percent: int | None = Field(50, alias="buf_free_run")
    buffering_mode: APACHE_KAFKA.BufferingMode | None = Field(APACHE_KAFKA.BufferingMode.default, alias="buf_mode")
    collecting: APACHE_KAFKA.Collecting | None = Field(APACHE_KAFKA.Collecting.auto, alias="coll_type")
    column_metadata_change_propagation: bool | None = Field(None, alias="auto_column_propagation")
    combinability_mode: APACHE_KAFKA.CombinabilityMode | None = Field(
        APACHE_KAFKA.CombinabilityMode.auto, alias="combinability"
    )
    conn_registry_key_chain_pem: str | None = Field(None, alias="conn_registry_key_chain_pem")
    conn_registry_key_password: str = Field(None, alias="conn_registry_key_password")
    conn_registry_key_pem: str | None = Field(None, alias="conn_registry_key_pem")
    conn_registry_keystore_location: str | None = Field(None, alias="conn_registry_keystore_location")
    conn_registry_keystore_password: str = Field(None, alias="conn_registry_keystore_password")
    conn_registry_keytab: str | None = Field(None, alias="conn_registry_keytab")
    conn_registry_password: str = Field(None, alias="conn_registry_password")
    conn_registry_principal_name: str | None = Field(None, alias="conn_registry_principal_name")
    conn_registry_truststore_pem: str | None = Field(None, alias="conn_registry_truststore_pem")
    conn_registry_username: str | None = Field(None, alias="conn_registry_username")
    conn_schema_registry_authentication: APACHE_KAFKA.ConnSchemaRegistryAuthentication | None = Field(
        APACHE_KAFKA.ConnSchemaRegistryAuthentication.none, alias="conn_schema_registry_authentication"
    )
    conn_schema_registry_secure: APACHE_KAFKA.ConnSchemaRegistrySecure | None = Field(
        APACHE_KAFKA.ConnSchemaRegistrySecure.none, alias="conn_schema_registry_secure"
    )
    consumer_group: str | None = Field(None, alias="consumer_group_name")
    continuous_mode: bool | None = Field(False, alias="continuous_mode")
    current_output_link_type: str = Field("PRIMARY", alias="currentOutputLinkType")
    db2_database_name: str | None = Field(None, alias="part_client_dbname")
    db2_instance_name: str | None = Field(None, alias="part_client_instance")
    db2_source_connection_required: str | None = Field("", alias="part_dbconnection")
    db2_table_name: str | None = Field(None, alias="part_table")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    disk_write_inc_ronly: int | None = Field(1048576, alias="disk_write_inc_ronly")
    disk_write_increment_bytes: int | None = Field(1048576, alias="disk_write_inc")
    ds_advanced_client_logging: bool | None = Field(False, alias="_advanced_client_logging")
    ds_advanced_kafka_config_options: bool | None = Field(False, alias="_advanced_kafka_config_options")
    ds_client_logging_level: APACHE_KAFKA.DSClientLoggingLevel | None = Field(
        APACHE_KAFKA.DSClientLoggingLevel.off, alias="_client_logging_level"
    )
    ds_consumer_group_name: str | None = Field(None, alias="_consumer_group_name")
    ds_isolation_level: APACHE_KAFKA.DSIsolationLevel | None = Field(
        APACHE_KAFKA.DSIsolationLevel.read_uncommitted, alias="_isolation_level"
    )
    ds_java_heap_size: int | None = Field(256, alias="_java._heap_size")
    ds_kafka_config_options: str | None = Field(None, alias="_kafka_config_options")
    ds_key_serializer_type: APACHE_KAFKA.DSKeySerializerType | None = Field(
        APACHE_KAFKA.DSKeySerializerType.string, alias="_key_serializer_type"
    )
    ds_max_messages: int | None = Field(None, alias="_max_messages")
    ds_max_poll_records: int | None = Field(100, alias="_max_poll_records")
    ds_reset_policy: APACHE_KAFKA.DSResetPolicy | None = Field(
        APACHE_KAFKA.DSResetPolicy.latest, alias="_reset_policy"
    )
    ds_start_offset: str | None = Field(None, alias="_start_offset")
    ds_stop_message: str | None = Field(None, alias="_stop_message")
    ds_timeout: int | None = Field(None, alias="_timeout")
    ds_timeout_after_last_message: int | None = Field(30, alias="_timeout_after_last_message")
    ds_use_datastage: bool | None = Field(True, alias="_use_datastage")
    ds_value_serializer_type: APACHE_KAFKA.DSValueSerializerType | None = Field(
        APACHE_KAFKA.DSValueSerializerType.string, alias="_value_serializer_type"
    )
    ds_warn_and_error_log: APACHE_KAFKA.DSWarnAndErrorLog | None = Field(
        APACHE_KAFKA.DSWarnAndErrorLog.log_as_informational, alias="_warn_and_error_log"
    )
    enable_flow_acp_control: bool = Field(True, alias="enableFlowAcpControl")
    enable_schemaless_design: bool = Field(False, alias="enableSchemalessDesign")
    end_of_data: bool | None = Field(False, alias="end_of_data")
    end_of_wave: bool | None = Field(False, alias="end_of_wave")
    execution_mode: APACHE_KAFKA.ExecutionMode | None = Field(
        APACHE_KAFKA.ExecutionMode.default_par, alias="execmode"
    )
    flow_dirty: str | None = Field("false", alias="flow_dirty")
    generate_unicode_type_columns: bool | None = Field(False, alias="generate_unicode_columns")
    heap_size: int | None = Field(256, alias="heap_size")
    hide: bool | None = Field(False, alias="hide")
    input_count: int | None = Field(0, alias="input_count")
    input_link_description: list | None = Field("", alias="inputLinkDescription")
    inputcol_properties: list | None = Field([], alias="inputcolProperties")
    isolation_level: APACHE_KAFKA.IsolationLevel | None = Field(
        APACHE_KAFKA.IsolationLevel.read_uncommitted, alias="isolation_level"
    )
    job_timeout_in_seconds: int | None = Field(30, alias="timeout")
    kafka_client_logging_level: APACHE_KAFKA.KafkaClientLoggingLevel | None = Field(
        APACHE_KAFKA.KafkaClientLoggingLevel.off, alias="kafka_client_logging_level"
    )
    kafka_config_options: str | None = Field(None, alias="kafka_config_options")
    kafka_start_offset: str | None = Field(None, alias="start_offset")
    kafka_warning_and_error_logs: APACHE_KAFKA.KafkaWarningAndErrorLogs | None = Field(
        APACHE_KAFKA.KafkaWarningAndErrorLogs.log_as_informational, alias="kafka_warning_and_error_logs"
    )
    key_cols_coll: list | None = Field([], alias="keyColsColl")
    key_cols_coll_ordered: list | None = Field([], alias="keyColsCollOrdered")
    key_cols_coll_robin: list | None = Field([], alias="keyColsCollRobin")
    key_cols_none: list | None = Field([], alias="keyColsNone")
    key_cols_part: list | None = Field([], alias="keyColsPart")
    key_serializer: APACHE_KAFKA.KeySerializer | None = Field(
        APACHE_KAFKA.KeySerializer.string, alias="key_serializer_type"
    )
    max_mem_buf_size_ronly: int | None = Field(3145728, alias="max_mem_buf_size_ronly")
    maximum_memory_buffer_size_bytes: int | None = Field(3145728, alias="max_mem_buf_size")
    messages_read_within_single_request: int | None = Field(100, alias="max_poll_records")
    output_acp_should_hide: bool = Field(True, alias="outputAcpShouldHide")
    output_count: int | None = Field(1, alias="output_count")
    output_link_description: list | None = Field("", alias="outputLinkDescription")
    outputcol_properties: list | None = Field([], alias="outputcolProperties")
    part_stable_coll: bool | None = Field(False, alias="part_stable_coll")
    part_stable_ordered: bool | None = Field(False, alias="part_stable_ordered")
    part_stable_roundrobin_coll: bool | None = Field(False, alias="part_stable_roundrobin_coll")
    part_unique_coll: bool | None = Field(False, alias="part_unique_coll")
    part_unique_ordered: bool | None = Field(False, alias="part_unique_ordered")
    part_unique_roundrobin_coll: bool | None = Field(False, alias="part_unique_roundrobin_coll")
    partition_type: APACHE_KAFKA.PartitionType | None = Field(APACHE_KAFKA.PartitionType.auto, alias="part_type")
    perform_sort: bool | None = Field(False, alias="perform_sort")
    perform_sort_coll: bool | None = Field(False, alias="perform_sort_coll")
    perform_sort_modulus: bool | None = Field(False, alias="perform_sort_modulus")
    preserve_partitioning: APACHE_KAFKA.PreservePartitioning | None = Field(
        APACHE_KAFKA.PreservePartitioning.default_propagate, alias="preserve"
    )
    queue_upper_bound_size_bytes: int | None = Field(0, alias="queue_upper_size")
    queue_upper_size_ronly: int | None = Field(0, alias="queue_upper_size_ronly")
    record_count: int | None = Field(0, alias="record_count")
    registry_trust_location: str | None = Field(None, alias="registry_trust_location")
    registry_trust_password: str | None = Field(None, alias="registry_trust_password")
    reset_policy: APACHE_KAFKA.ResetPolicy | None = Field(APACHE_KAFKA.ResetPolicy.earliest, alias="reset_policy")
    row_limit: int | None = Field(None, alias="row_limit")
    runtime_column_propagation: bool | None = Field(None, alias="runtime_column_propagation")
    schema_registry_url: str | None = Field(None, alias="conn_schema_registry_url")
    show_coll_type: int | None = Field(0, alias="showCollType")
    show_part_type: int | None = Field(1, alias="showPartType")
    show_sort_options: int | None = Field(0, alias="showSortOptions")
    sort_instructions: str | None = Field("", alias="sortInstructions")
    sort_instructions_text: str | None = Field("", alias="sortInstructionsText")
    sorting_key: APACHE_KAFKA.KeyColSelect | None = Field(APACHE_KAFKA.KeyColSelect.default, alias="keyColSelect")
    stable: bool | None = Field(None, alias="part_stable")
    stage_description: list | None = Field("", alias="stageDescription")
    stop_message_pattern: str | None = Field(None, alias="stop_message")
    time_interval: int | None = Field(0, alias="time_interval")
    topic_name: str = Field(None, alias="topic_name")
    total_number_of_messages: int | None = Field(100, alias="max_messages")
    unique: bool | None = Field(None, alias="part_unique")
    value_serializer: APACHE_KAFKA.ValueSerializer | None = Field(
        APACHE_KAFKA.ValueSerializer.string, alias="value_serializer_type"
    )

    def validate_parameters(self) -> tuple[set, set]:
        include = set()
        exclude = set()

        (
            include.add("preserve_partitioning")
            if (self.output_count and self.output_count > 0)
            else exclude.add("preserve_partitioning")
        )
        return include, exclude

    def validate_source(self) -> tuple[set, set]:
        include = set()
        exclude = set()

        include.add("registry_trust_password") if (()) else exclude.add("registry_trust_password")
        (
            include.add("kafka_warning_and_error_logs")
            if (

                    (
                        (
                            self.kafka_client_logging_level
                            and (
                                (
                                    hasattr(self.kafka_client_logging_level, "value")
                                    and self.kafka_client_logging_level.value == "debug"
                                )
                                or (self.kafka_client_logging_level == "debug")
                            )
                        )
                    )
                    or (

                            self.kafka_client_logging_level
                            and (
                                (
                                    hasattr(self.kafka_client_logging_level, "value")
                                    and self.kafka_client_logging_level.value == "error"
                                )
                                or (self.kafka_client_logging_level == "error")
                            )

                    )
                    or (

                            self.kafka_client_logging_level
                            and (
                                (
                                    hasattr(self.kafka_client_logging_level, "value")
                                    and self.kafka_client_logging_level.value == "fatal"
                                )
                                or (self.kafka_client_logging_level == "fatal")
                            )

                    )
                    or (

                            self.kafka_client_logging_level
                            and (
                                (
                                    hasattr(self.kafka_client_logging_level, "value")
                                    and self.kafka_client_logging_level.value == "info"
                                )
                                or (self.kafka_client_logging_level == "info")
                            )

                    )
                    or (

                            self.kafka_client_logging_level
                            and (
                                (
                                    hasattr(self.kafka_client_logging_level, "value")
                                    and self.kafka_client_logging_level.value == "trace"
                                )
                                or (self.kafka_client_logging_level == "trace")
                            )

                    )
                    or (

                            self.kafka_client_logging_level
                            and (
                                (
                                    hasattr(self.kafka_client_logging_level, "value")
                                    and self.kafka_client_logging_level.value == "warn"
                                )
                                or (self.kafka_client_logging_level == "warn")
                            )

                    )

            )
            else exclude.add("kafka_warning_and_error_logs")
        )
        include.add("time_interval") if (self.record_count == "0.0") else exclude.add("time_interval")
        (
            include.add("job_timeout_in_seconds")
            if (not self.continuous_mode)
            else exclude.add("job_timeout_in_seconds")
        )
        include.add("end_of_data") if (self.end_of_wave) else exclude.add("end_of_data")
        include.add("registry_trust_location") if (()) else exclude.add("registry_trust_location")
        (
            include.add("stop_message_pattern")
            if (self.continuous_mode)
            else exclude.add("stop_message_pattern")
        )
        (
            include.add("ds_client_logging_level")
            if (self.ds_advanced_client_logging)
            else exclude.add("ds_client_logging_level")
        )
        (
            include.add("ds_warn_and_error_log")
            if (
                (((self.ds_advanced_client_logging)))
                and (
                    ((self.ds_client_logging_level == "debug"))
                    or (self.ds_client_logging_level == "error")
                    or (self.ds_client_logging_level == "fatal")
                    or (self.ds_client_logging_level == "info")
                    or (self.ds_client_logging_level == "trace")
                    or (self.ds_client_logging_level == "warn")
                )
            )
            else exclude.add("ds_warn_and_error_log")
        )
        (
            include.add("ds_kafka_config_options")
            if (self.ds_advanced_kafka_config_options)
            else exclude.add("ds_kafka_config_options")
        )
        (
            include.add("registry_trust_password")
            if (((((()) or (())))) and (self.ds_use_datastage))
            else exclude.add("registry_trust_password")
        )
        (
            include.add("ds_client_logging_level")
            if (

                    (
                        ((self.ds_advanced_client_logging))
                        or ((self.ds_advanced_client_logging and "#" in str(self.ds_advanced_client_logging)))
                    )
                    and (not self.ds_use_datastage)

            )
            else exclude.add("ds_client_logging_level")
        )
        (
            include.add("ds_warn_and_error_log")
            if (

                    (
                        (
                            (
                                ((self.ds_advanced_client_logging))
                                or ((self.ds_advanced_client_logging and "#" in str(self.ds_advanced_client_logging)))
                            )
                            and (
                                ((self.ds_client_logging_level == "debug"))
                                or ((self.ds_client_logging_level == "error"))
                                or ((self.ds_client_logging_level == "fatal"))
                                or ((self.ds_client_logging_level == "info"))
                                or ((self.ds_client_logging_level == "trace"))
                                or ((self.ds_client_logging_level == "warn"))
                                or ((self.ds_client_logging_level and "#" in str(self.ds_client_logging_level)))
                            )
                        )
                    )
                    and (not self.ds_use_datastage)

            )
            else exclude.add("ds_warn_and_error_log")
        )
        (
            include.add("kafka_warning_and_error_logs")
            if (

                    (
                        (
                            (
                                self.kafka_client_logging_level
                                and (
                                    (
                                        hasattr(self.kafka_client_logging_level, "value")
                                        and self.kafka_client_logging_level.value == "debug"
                                    )
                                    or (self.kafka_client_logging_level == "debug")
                                )
                            )
                        )
                        or (
                            (
                                self.kafka_client_logging_level
                                and (
                                    (
                                        hasattr(self.kafka_client_logging_level, "value")
                                        and self.kafka_client_logging_level.value == "error"
                                    )
                                    or (self.kafka_client_logging_level == "error")
                                )
                            )
                        )
                        or (
                            (
                                self.kafka_client_logging_level
                                and (
                                    (
                                        hasattr(self.kafka_client_logging_level, "value")
                                        and self.kafka_client_logging_level.value == "fatal"
                                    )
                                    or (self.kafka_client_logging_level == "fatal")
                                )
                            )
                        )
                        or (
                            (
                                self.kafka_client_logging_level
                                and (
                                    (
                                        hasattr(self.kafka_client_logging_level, "value")
                                        and self.kafka_client_logging_level.value == "info"
                                    )
                                    or (self.kafka_client_logging_level == "info")
                                )
                            )
                        )
                        or (
                            (
                                self.kafka_client_logging_level
                                and (
                                    (
                                        hasattr(self.kafka_client_logging_level, "value")
                                        and self.kafka_client_logging_level.value == "trace"
                                    )
                                    or (self.kafka_client_logging_level == "trace")
                                )
                            )
                        )
                        or (
                            (
                                self.kafka_client_logging_level
                                and (
                                    (
                                        hasattr(self.kafka_client_logging_level, "value")
                                        and self.kafka_client_logging_level.value == "warn"
                                    )
                                    or (self.kafka_client_logging_level == "warn")
                                )
                            )
                        )
                        or (
                            (
                                self.kafka_client_logging_level
                                and (
                                    (
                                        hasattr(self.kafka_client_logging_level, "value")
                                        and self.kafka_client_logging_level.value
                                        and "#" in str(self.kafka_client_logging_level.value)
                                    )
                                    or ("#" in str(self.kafka_client_logging_level))
                                )
                            )
                        )
                    )
                    and (self.ds_use_datastage)

            )
            else exclude.add("kafka_warning_and_error_logs")
        )
        (
            include.add("time_interval")
            if (

                    (((self.record_count == 0)) or ((self.record_count and "#" in str(self.record_count))))
                    and (self.ds_use_datastage)

            )
            else exclude.add("time_interval")
        )
        (
            include.add("job_timeout_in_seconds")
            if (

                    (((not self.continuous_mode)) or ((self.continuous_mode and "#" in str(self.continuous_mode))))
                    and (self.ds_use_datastage)

            )
            else exclude.add("job_timeout_in_seconds")
        )
        (
            include.add("end_of_data")
            if (

                    (((self.end_of_wave)) or ((self.end_of_wave and "#" in str(self.end_of_wave))))
                    and (self.ds_use_datastage)

            )
            else exclude.add("end_of_data")
        )
        (
            include.add("registry_trust_location")
            if (((((()) or (())))) and (self.ds_use_datastage))
            else exclude.add("registry_trust_location")
        )
        (
            include.add("ds_kafka_config_options")
            if (

                    (
                        ((self.ds_advanced_kafka_config_options))
                        or (
                            (
                                self.ds_advanced_kafka_config_options
                                and "#" in str(self.ds_advanced_kafka_config_options)
                            )
                        )
                    )
                    and (not self.ds_use_datastage)

            )
            else exclude.add("ds_kafka_config_options")
        )
        (
            include.add("stop_message_pattern")
            if (

                    (((self.continuous_mode)) or ((self.continuous_mode and "#" in str(self.continuous_mode))))
                    and (self.ds_use_datastage)

            )
            else exclude.add("stop_message_pattern")
        )
        include.add("ds_java_heap_size") if (not self.ds_use_datastage) else exclude.add("ds_java_heap_size")
        (
            include.add("advanced_kafka_config_options")
            if (self.ds_use_datastage)
            else exclude.add("advanced_kafka_config_options")
        )
        include.add("end_of_wave") if (self.ds_use_datastage) else exclude.add("end_of_wave")
        (
            include.add("kafka_config_options")
            if (self.ds_use_datastage)
            else exclude.add("kafka_config_options")
        )
        include.add("ds_max_messages") if (not self.ds_use_datastage) else exclude.add("ds_max_messages")
        (
            include.add("messages_read_within_single_request")
            if (self.ds_use_datastage)
            else exclude.add("messages_read_within_single_request")
        )
        include.add("heap_size") if (self.ds_use_datastage) else exclude.add("heap_size")
        include.add("reset_policy") if (self.ds_use_datastage) else exclude.add("reset_policy")
        include.add("ds_timeout") if (not self.ds_use_datastage) else exclude.add("ds_timeout")
        (
            include.add("kafka_client_logging_level")
            if (self.ds_use_datastage)
            else exclude.add("kafka_client_logging_level")
        )
        include.add("continuous_mode") if (self.ds_use_datastage) else exclude.add("continuous_mode")
        include.add("ds_max_poll_records") if (not self.ds_use_datastage) else exclude.add("ds_max_poll_records")
        include.add("key_serializer") if (self.ds_use_datastage) else exclude.add("key_serializer")
        (
            include.add("ds_advanced_kafka_config_options")
            if (not self.ds_use_datastage)
            else exclude.add("ds_advanced_kafka_config_options")
        )
        (
            include.add("ds_value_serializer_type")
            if (not self.ds_use_datastage)
            else exclude.add("ds_value_serializer_type")
        )
        (
            include.add("ds_timeout_after_last_message")
            if (not self.ds_use_datastage)
            else exclude.add("ds_timeout_after_last_message")
        )
        include.add("record_count") if (self.ds_use_datastage) else exclude.add("record_count")
        include.add("ds_isolation_level") if (not self.ds_use_datastage) else exclude.add("ds_isolation_level")
        (
            include.add("ds_advanced_client_logging")
            if (not self.ds_use_datastage)
            else exclude.add("ds_advanced_client_logging")
        )
        include.add("consumer_group") if (self.ds_use_datastage) else exclude.add("consumer_group")
        (
            include.add("total_number_of_messages")
            if (self.ds_use_datastage)
            else exclude.add("total_number_of_messages")
        )
        include.add("ds_reset_policy") if (not self.ds_use_datastage) else exclude.add("ds_reset_policy")
        include.add("value_serializer") if (self.ds_use_datastage) else exclude.add("value_serializer")
        (
            include.add("generate_unicode_type_columns")
            if (not self.ds_use_datastage)
            else exclude.add("generate_unicode_type_columns")
        )
        include.add("isolation_level") if (self.ds_use_datastage) else exclude.add("isolation_level")
        include.add("ds_start_offset") if (not self.ds_use_datastage) else exclude.add("ds_start_offset")
        (
            include.add("ds_key_serializer_type")
            if (not self.ds_use_datastage)
            else exclude.add("ds_key_serializer_type")
        )
        (
            include.add("ds_consumer_group_name")
            if (not self.ds_use_datastage)
            else exclude.add("ds_consumer_group_name")
        )
        include.add("row_limit") if (not self.ds_use_datastage) else exclude.add("row_limit")
        include.add("ds_stop_message") if (not self.ds_use_datastage) else exclude.add("ds_stop_message")
        include.add("kafka_start_offset") if (self.ds_use_datastage) else exclude.add("kafka_start_offset")
        (
            include.add("maximum_memory_buffer_size_bytes")
            if (self.buffering_mode != "nobuffer")
            else exclude.add("maximum_memory_buffer_size_bytes")
        )
        (
            include.add("buffer_free_run_percent")
            if (self.buffering_mode != "nobuffer")
            else exclude.add("buffer_free_run_percent")
        )
        (
            include.add("queue_upper_bound_size_bytes")
            if (self.buffering_mode != "nobuffer")
            else exclude.add("queue_upper_bound_size_bytes")
        )
        (
            include.add("disk_write_increment_bytes")
            if (self.buffering_mode != "nobuffer")
            else exclude.add("disk_write_increment_bytes")
        )
        (
            include.add("runtime_column_propagation")
            if (not self.enable_schemaless_design)
            else exclude.add("runtime_column_propagation")
        )
        (
            include.add("column_metadata_change_propagation")
            if (not self.output_acp_should_hide)
            else exclude.add("column_metadata_change_propagation")
        )
        (
            include.add("max_mem_buf_size_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("max_mem_buf_size_ronly")
        )
        (
            include.add("buf_free_run_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("buf_free_run_ronly")
        )
        (
            include.add("queue_upper_size_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("queue_upper_size_ronly")
        )
        (
            include.add("disk_write_inc_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("disk_write_inc_ronly")
        )
        (
            include.add("stable")
            if (

                    ((self.show_part_type))
                    and (self.partition_type != "auto")
                    and (self.partition_type != "db2connector")
                    and (self.show_sort_options)

            )
            else exclude.add("stable")
        )
        (
            include.add("unique")
            if (

                    ((self.show_part_type))
                    and (self.partition_type != "auto")
                    and (self.partition_type != "db2connector")
                    and (self.show_sort_options)

            )
            else exclude.add("unique")
        )
        (
            include.add("key_cols_part")
            if (
                (
                    (
                        ((self.show_part_type))
                        and ((not self.show_coll_type))
                        and ((self.partition_type != "auto"))
                        and ((self.partition_type != "db2connector"))
                        and ((self.partition_type != "modulus"))
                    )
                )
                or (

                        ((self.show_part_type))
                        and (not self.show_coll_type)
                        and (self.partition_type == "modulus")
                        and (self.perform_sort_modulus)

                )
            )
            else exclude.add("key_cols_part")
        )
        (
            include.add("db2_source_connection_required")
            if (((self.partition_type == "db2part")) and (self.show_part_type))
            else exclude.add("db2_source_connection_required")
        )
        (
            include.add("db2_database_name")
            if (((self.partition_type == "db2part")) and (self.show_part_type))
            else exclude.add("db2_database_name")
        )
        (
            include.add("db2_instance_name")
            if (((self.partition_type == "db2part")) and (self.show_part_type))
            else exclude.add("db2_instance_name")
        )
        (
            include.add("db2_table_name")
            if (((self.partition_type == "db2part")) and (self.show_part_type))
            else exclude.add("db2_table_name")
        )
        (
            include.add("perform_sort")
            if (

                    ((self.show_part_type))
                    and (((self.partition_type == "hash")) or (self.partition_type == "range"))

            )
            else exclude.add("perform_sort")
        )
        (
            include.add("perform_sort_modulus")
            if (((self.show_part_type)) and (self.partition_type == "modulus"))
            else exclude.add("perform_sort_modulus")
        )
        (
            include.add("sorting_key")
            if (

                    ((self.show_part_type))
                    and (self.partition_type == "modulus")
                    and (not self.perform_sort_modulus)

            )
            else exclude.add("sorting_key")
        )
        (
            include.add("sort_instructions")
            if (

                    ((self.show_part_type))
                    and (
                        ((self.partition_type == "db2part"))
                        or (self.partition_type == "entire")
                        or (self.partition_type == "random")
                        or (self.partition_type == "roundrobin")
                        or (self.partition_type == "same")
                    )

            )
            else exclude.add("sort_instructions")
        )
        (
            include.add("sort_instructions_text")
            if (

                    ((self.show_part_type))
                    and (
                        ((self.partition_type == "db2part"))
                        or (self.partition_type == "entire")
                        or (self.partition_type == "random")
                        or (self.partition_type == "roundrobin")
                        or (self.partition_type == "same")
                    )

            )
            else exclude.add("sort_instructions_text")
        )
        include.add("collecting") if (self.show_coll_type) else exclude.add("collecting")
        include.add("partition_type") if (self.show_part_type) else exclude.add("partition_type")
        (
            include.add("perform_sort_coll")
            if (
                (
                    (
                        ((self.show_coll_type))
                        and (
                            ((self.collecting == "ordered"))
                            or ((self.collecting == "roundrobin_coll"))
                            or ((self.collecting == "sortmerge"))
                        )
                    )
                )
                or (((not self.show_part_type)) and (not self.show_coll_type))
            )
            else exclude.add("perform_sort_coll")
        )
        (
            include.add("key_cols_coll")
            if (

                    ((self.show_coll_type))
                    and (not self.show_part_type)
                    and (self.collecting != "auto")
                    and (((self.collecting == "sortmerge")) or (self.perform_sort_coll))

            )
            else exclude.add("key_cols_coll")
        )
        (
            include.add("key_cols_none")
            if (

                    ((not self.show_part_type))
                    and (not self.show_coll_type)
                    and (self.perform_sort_coll)

            )
            else exclude.add("key_cols_none")
        )
        (
            include.add("part_stable_coll")
            if (

                    (((self.perform_sort_coll)))
                    and (
                        (
                            (
                                ((not self.show_part_type))
                                and ((self.show_coll_type))
                                and ((self.collecting != "auto"))
                                and ((self.show_sort_options))
                            )
                        )
                        or (

                                ((not self.show_part_type))
                                and (not self.show_coll_type)
                                and (self.show_sort_options)

                        )
                    )

            )
            else exclude.add("part_stable_coll")
        )
        (
            include.add("part_unique_coll")
            if (

                    (((self.perform_sort_coll)))
                    and (
                        (
                            (
                                ((not self.show_part_type))
                                and ((self.show_coll_type))
                                and ((self.collecting != "auto"))
                                and ((self.show_sort_options))
                            )
                        )
                        or (

                                ((not self.show_part_type))
                                and (not self.show_coll_type)
                                and (self.show_sort_options)

                        )
                    )

            )
            else exclude.add("part_unique_coll")
        )
        (
            include.add("conn_registry_key_pem")
            if (

                    (
                        ((self.conn_schema_registry_secure == "ssl"))
                        or ((self.conn_schema_registry_secure and "#" in str(self.conn_schema_registry_secure)))
                    )
                    and ((()) or (()))
                    and (not self.defer_credentials)
                    and (())

            )
            else exclude.add("conn_registry_key_pem")
        )
        (
            include.add("conn_schema_registry_authentication")
            if ((()) and (((()) or (())) or ((()) or (()))))
            else exclude.add("conn_schema_registry_authentication")
        )
        (
            include.add("conn_registry_keystore_location")
            if (

                    (())
                    and (
                        (
                            (
                                ((()) or (()))
                                and (
                                    ((self.conn_schema_registry_secure == "ssl"))
                                    or (
                                        (
                                            self.conn_schema_registry_secure
                                            and "#" in str(self.conn_schema_registry_secure)
                                        )
                                    )
                                )
                                and ((()) or (()))
                            )
                        )
                        or (

                                ((()) or (()))
                                and (
                                    ((self.conn_schema_registry_secure == "ssl"))
                                    or (

                                            self.conn_schema_registry_secure
                                            and "#" in str(self.conn_schema_registry_secure)

                                    )
                                )
                                and ((()) or (()))

                        )
                    )

            )
            else exclude.add("conn_registry_keystore_location")
        )
        (
            include.add("conn_registry_password")
            if (

                    (
                        ((self.conn_schema_registry_authentication == "user_credentials"))
                        or (
                            (
                                self.conn_schema_registry_authentication
                                and "#" in str(self.conn_schema_registry_authentication)
                            )
                        )
                    )
                    and ((()) or (()))
                    and (not self.defer_credentials)
                    and (())

            )
            else exclude.add("conn_registry_password")
        )
        (
            include.add("conn_registry_keytab")
            if (

                    (())
                    and (
                        (
                            (
                                (
                                    ((self.conn_schema_registry_secure == "kerberos"))
                                    or (
                                        (
                                            self.conn_schema_registry_secure
                                            and "#" in str(self.conn_schema_registry_secure)
                                        )
                                    )
                                )
                                and ((()) or (()))
                                and ((not self.defer_credentials))
                            )
                        )
                        or (

                                (
                                    ((self.conn_schema_registry_secure == "kerberos"))
                                    or (
                                        (
                                            self.conn_schema_registry_secure
                                            and "#" in str(self.conn_schema_registry_secure)
                                        )
                                    )
                                )
                                and ((()) or (()))
                                and ((()) or (()))
                                and (not self.defer_credentials)

                        )
                    )

            )
            else exclude.add("conn_registry_keytab")
        )
        (
            include.add("conn_registry_principal_name")
            if (

                    (
                        ((self.conn_schema_registry_secure == "kerberos"))
                        or ((self.conn_schema_registry_secure and "#" in str(self.conn_schema_registry_secure)))
                    )
                    and ((()) or (()))
                    and (not self.defer_credentials)
                    and (())

            )
            else exclude.add("conn_registry_principal_name")
        )
        (
            include.add("conn_registry_key_chain_pem")
            if (

                    (
                        ((self.conn_schema_registry_secure == "ssl"))
                        or ((self.conn_schema_registry_secure and "#" in str(self.conn_schema_registry_secure)))
                    )
                    and ((()) or (()))
                    and (not self.defer_credentials)
                    and (())

            )
            else exclude.add("conn_registry_key_chain_pem")
        )
        (
            include.add("conn_registry_keystore_password")
            if (

                    (())
                    and (
                        (
                            (
                                ((()) or (()))
                                and (
                                    ((self.conn_schema_registry_secure == "ssl"))
                                    or (
                                        (
                                            self.conn_schema_registry_secure
                                            and "#" in str(self.conn_schema_registry_secure)
                                        )
                                    )
                                )
                                and ((()) or (()))
                            )
                        )
                        or (

                                ((()) or (()))
                                and (
                                    ((self.conn_schema_registry_secure == "ssl"))
                                    or (

                                            self.conn_schema_registry_secure
                                            and "#" in str(self.conn_schema_registry_secure)

                                    )
                                )
                                and ((()) or (()))

                        )
                    )

            )
            else exclude.add("conn_registry_keystore_password")
        )
        include.add("schema_registry_url") if ((()) and ((()) or (()))) else exclude.add("schema_registry_url")
        (
            include.add("conn_registry_username")
            if (

                    (
                        ((self.conn_schema_registry_authentication == "user_credentials"))
                        or (
                            (
                                self.conn_schema_registry_authentication
                                and "#" in str(self.conn_schema_registry_authentication)
                            )
                        )
                    )
                    and ((()) or (()))
                    and (not self.defer_credentials)
                    and (())

            )
            else exclude.add("conn_registry_username")
        )
        (
            include.add("conn_registry_truststore_pem")
            if (

                    (
                        ((self.conn_schema_registry_secure == "kerberos"))
                        or ((self.conn_schema_registry_secure == "ssl"))
                        or ((self.conn_schema_registry_secure and "#" in str(self.conn_schema_registry_secure)))
                    )
                    and ((()) or (()))
                    and (not self.defer_credentials)
                    and (())

            )
            else exclude.add("conn_registry_truststore_pem")
        )
        (
            include.add("conn_schema_registry_secure")
            if ((()) and (((((()) or (())))) or ((()) or (()))))
            else exclude.add("conn_schema_registry_secure")
        )
        (
            include.add("conn_registry_key_password")
            if (

                    (
                        ((self.conn_schema_registry_secure == "ssl"))
                        or ((self.conn_schema_registry_secure and "#" in str(self.conn_schema_registry_secure)))
                    )
                    and ((()) or (()))
                    and (not self.defer_credentials)
                    and (())

            )
            else exclude.add("conn_registry_key_password")
        )
        include.add("schema_registry_url") if (()) else exclude.add("schema_registry_url")
        (
            include.add("conn_schema_registry_authentication")
            if (())
            else exclude.add("conn_schema_registry_authentication")
        )
        include.add("conn_registry_username") if (()) else exclude.add("conn_registry_username")
        include.add("conn_registry_password") if (()) else exclude.add("conn_registry_password")
        include.add("conn_schema_registry_secure") if (()) else exclude.add("conn_schema_registry_secure")
        include.add("conn_registry_principal_name") if (()) else exclude.add("conn_registry_principal_name")
        include.add("conn_registry_keytab") if (()) else exclude.add("conn_registry_keytab")
        include.add("conn_registry_truststore_pem") if (()) else exclude.add("conn_registry_truststore_pem")
        include.add("conn_registry_key_pem") if (()) else exclude.add("conn_registry_key_pem")
        include.add("conn_registry_key_chain_pem") if (()) else exclude.add("conn_registry_key_chain_pem")
        include.add("conn_registry_key_password") if (()) else exclude.add("conn_registry_key_password")
        include.add("conn_registry_keystore_location") if (()) else exclude.add("conn_registry_keystore_location")
        include.add("conn_registry_keystore_password") if (()) else exclude.add("conn_registry_keystore_password")
        include.add("defer_credentials") if (()) else exclude.add("defer_credentials")
        (
            include.add("end_of_data")
            if (self.end_of_wave and "true" in str(self.end_of_wave))
            else exclude.add("end_of_data")
        )
        (
            include.add("ds_client_logging_level")
            if (self.ds_advanced_client_logging and "true" in str(self.ds_advanced_client_logging))
            else exclude.add("ds_client_logging_level")
        )
        (
            include.add("ds_value_serializer_type")
            if (
                self.ds_key_serializer_type
                and "avro" in str(self.ds_key_serializer_type)
                and self.ds_key_serializer_type
                and "avro_to_json" in str(self.ds_key_serializer_type)
            )
            else exclude.add("ds_value_serializer_type")
        )
        (
            include.add("kafka_warning_and_error_logs")
            if (
                self.kafka_client_logging_level
                and (
                    (
                        hasattr(self.kafka_client_logging_level, "value")
                        and self.kafka_client_logging_level.value
                        and "debug" in str(self.kafka_client_logging_level.value)
                    )
                    or ("debug" in str(self.kafka_client_logging_level))
                )
                or self.kafka_client_logging_level
                and (
                    (
                        hasattr(self.kafka_client_logging_level, "value")
                        and self.kafka_client_logging_level.value
                        and "error" in str(self.kafka_client_logging_level.value)
                    )
                    or ("error" in str(self.kafka_client_logging_level))
                )
                or self.kafka_client_logging_level
                and (
                    (
                        hasattr(self.kafka_client_logging_level, "value")
                        and self.kafka_client_logging_level.value
                        and "fatal" in str(self.kafka_client_logging_level.value)
                    )
                    or ("fatal" in str(self.kafka_client_logging_level))
                )
                or self.kafka_client_logging_level
                and (
                    (
                        hasattr(self.kafka_client_logging_level, "value")
                        and self.kafka_client_logging_level.value
                        and "info" in str(self.kafka_client_logging_level.value)
                    )
                    or ("info" in str(self.kafka_client_logging_level))
                )
                or self.kafka_client_logging_level
                and (
                    (
                        hasattr(self.kafka_client_logging_level, "value")
                        and self.kafka_client_logging_level.value
                        and "trace" in str(self.kafka_client_logging_level.value)
                    )
                    or ("trace" in str(self.kafka_client_logging_level))
                )
                or self.kafka_client_logging_level
                and (
                    (
                        hasattr(self.kafka_client_logging_level, "value")
                        and self.kafka_client_logging_level.value
                        and "warn" in str(self.kafka_client_logging_level.value)
                    )
                    or ("warn" in str(self.kafka_client_logging_level))
                )
            )
            else exclude.add("kafka_warning_and_error_logs")
        )
        include.add("time_interval") if (self.record_count == "0") else exclude.add("time_interval")
        (
            include.add("value_serializer")
            if (
                self.key_serializer
                and (
                    (
                        hasattr(self.key_serializer, "value")
                        and self.key_serializer.value
                        and "avro" in str(self.key_serializer.value)
                    )
                    or ("avro" in str(self.key_serializer))
                )
                and self.key_serializer
                and (
                    (
                        hasattr(self.key_serializer, "value")
                        and self.key_serializer.value
                        and "avro_to_json" in str(self.key_serializer.value)
                    )
                    or ("avro_to_json" in str(self.key_serializer))
                )
            )
            else exclude.add("value_serializer")
        )
        (
            include.add("ds_warn_and_error_log")
            if (self.ds_advanced_client_logging and "true" in str(self.ds_advanced_client_logging))
            and (
                self.ds_client_logging_level
                and "debug" in str(self.ds_client_logging_level)
                and self.ds_client_logging_level
                and "error" in str(self.ds_client_logging_level)
                and self.ds_client_logging_level
                and "fatal" in str(self.ds_client_logging_level)
                and self.ds_client_logging_level
                and "info" in str(self.ds_client_logging_level)
                and self.ds_client_logging_level
                and "trace" in str(self.ds_client_logging_level)
                and self.ds_client_logging_level
                and "warn" in str(self.ds_client_logging_level)
            )
            else exclude.add("ds_warn_and_error_log")
        )
        (
            include.add("ds_kafka_config_options")
            if (self.ds_advanced_kafka_config_options and "true" in str(self.ds_advanced_kafka_config_options))
            else exclude.add("ds_kafka_config_options")
        )
        (
            include.add("stop_message_pattern")
            if (self.continuous_mode and "true" in str(self.continuous_mode))
            else exclude.add("stop_message_pattern")
        )
        (
            include.add("job_timeout_in_seconds")
            if (self.continuous_mode and "false" in str(self.continuous_mode))
            else exclude.add("job_timeout_in_seconds")
        )
        return include, exclude

    def validate_target(self) -> tuple[set, set]:
        include = set()
        exclude = set()

        return include, exclude

    def get_source_props(self) -> dict:
        include, exclude = self.validate_source()
        props = {
            "advanced_kafka_config_options",
            "buf_free_run_ronly",
            "buf_mode_ronly",
            "buffer_free_run_percent",
            "buffering_mode",
            "collecting",
            "column_metadata_change_propagation",
            "combinability_mode",
            "conn_registry_key_chain_pem",
            "conn_registry_key_password",
            "conn_registry_key_pem",
            "conn_registry_keystore_location",
            "conn_registry_keystore_password",
            "conn_registry_keytab",
            "conn_registry_password",
            "conn_registry_principal_name",
            "conn_registry_truststore_pem",
            "conn_registry_username",
            "conn_schema_registry_authentication",
            "conn_schema_registry_secure",
            "consumer_group",
            "continuous_mode",
            "current_output_link_type",
            "db2_database_name",
            "db2_instance_name",
            "db2_source_connection_required",
            "db2_table_name",
            "disk_write_inc_ronly",
            "disk_write_increment_bytes",
            "ds_advanced_client_logging",
            "ds_advanced_kafka_config_options",
            "ds_client_logging_level",
            "ds_consumer_group_name",
            "ds_isolation_level",
            "ds_java_heap_size",
            "ds_kafka_config_options",
            "ds_key_serializer_type",
            "ds_max_messages",
            "ds_max_poll_records",
            "ds_reset_policy",
            "ds_start_offset",
            "ds_stop_message",
            "ds_timeout",
            "ds_timeout_after_last_message",
            "ds_use_datastage",
            "ds_value_serializer_type",
            "ds_warn_and_error_log",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "end_of_data",
            "end_of_wave",
            "execution_mode",
            "flow_dirty",
            "generate_unicode_type_columns",
            "heap_size",
            "hide",
            "input_count",
            "input_link_description",
            "inputcol_properties",
            "isolation_level",
            "job_timeout_in_seconds",
            "kafka_client_logging_level",
            "kafka_config_options",
            "kafka_start_offset",
            "kafka_warning_and_error_logs",
            "key_cols_coll",
            "key_cols_coll_ordered",
            "key_cols_coll_robin",
            "key_cols_none",
            "key_cols_part",
            "key_serializer",
            "max_mem_buf_size_ronly",
            "maximum_memory_buffer_size_bytes",
            "messages_read_within_single_request",
            "output_acp_should_hide",
            "output_count",
            "output_link_description",
            "outputcol_properties",
            "part_stable_coll",
            "part_stable_ordered",
            "part_stable_roundrobin_coll",
            "part_unique_coll",
            "part_unique_ordered",
            "part_unique_roundrobin_coll",
            "partition_type",
            "perform_sort",
            "perform_sort_coll",
            "perform_sort_modulus",
            "preserve_partitioning",
            "queue_upper_bound_size_bytes",
            "queue_upper_size_ronly",
            "record_count",
            "registry_trust_location",
            "registry_trust_password",
            "reset_policy",
            "row_limit",
            "runtime_column_propagation",
            "schema_registry_url",
            "show_coll_type",
            "show_part_type",
            "show_sort_options",
            "sort_instructions",
            "sort_instructions_text",
            "sorting_key",
            "stable",
            "stage_description",
            "stop_message_pattern",
            "time_interval",
            "topic_name",
            "total_number_of_messages",
            "unique",
            "value_serializer",
        }
        required = {
            "consumer_group",
            "current_output_link_type",
            "ds_consumer_group_name",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "kafka_server_host_name",
            "output_acp_should_hide",
            "topic_name",
        }
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_target_props(self) -> dict:
        include, exclude = self.validate_target()
        props = {
            "advanced_kafka_config_options",
            "buf_free_run_ronly",
            "buf_mode_ronly",
            "buffer_free_run_percent",
            "buffering_mode",
            "collecting",
            "column_metadata_change_propagation",
            "combinability_mode",
            "conn_registry_key_chain_pem",
            "conn_registry_key_password",
            "conn_registry_key_pem",
            "conn_registry_keystore_location",
            "conn_registry_keystore_password",
            "conn_registry_keytab",
            "conn_registry_password",
            "conn_registry_principal_name",
            "conn_registry_truststore_pem",
            "conn_registry_username",
            "conn_schema_registry_authentication",
            "conn_schema_registry_secure",
            "current_output_link_type",
            "db2_database_name",
            "db2_instance_name",
            "db2_source_connection_required",
            "db2_table_name",
            "disk_write_inc_ronly",
            "disk_write_increment_bytes",
            "ds_advanced_client_logging",
            "ds_advanced_kafka_config_options",
            "ds_client_logging_level",
            "ds_java_heap_size",
            "ds_kafka_config_options",
            "ds_key_serializer_type",
            "ds_use_datastage",
            "ds_value_serializer_type",
            "ds_warn_and_error_log",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "execution_mode",
            "flow_dirty",
            "heap_size",
            "hide",
            "input_count",
            "input_link_description",
            "inputcol_properties",
            "kafka_client_logging_level",
            "kafka_config_options",
            "kafka_warning_and_error_logs",
            "key_cols_coll",
            "key_cols_coll_ordered",
            "key_cols_coll_robin",
            "key_cols_none",
            "key_cols_part",
            "key_serializer",
            "max_mem_buf_size_ronly",
            "maximum_memory_buffer_size_bytes",
            "output_acp_should_hide",
            "output_count",
            "output_link_description",
            "outputcol_properties",
            "part_stable_coll",
            "part_stable_ordered",
            "part_stable_roundrobin_coll",
            "part_unique_coll",
            "part_unique_ordered",
            "part_unique_roundrobin_coll",
            "partition_type",
            "perform_sort",
            "perform_sort_coll",
            "perform_sort_modulus",
            "preserve_partitioning",
            "queue_upper_bound_size_bytes",
            "queue_upper_size_ronly",
            "registry_trust_location",
            "registry_trust_password",
            "runtime_column_propagation",
            "schema_registry_url",
            "show_coll_type",
            "show_part_type",
            "show_sort_options",
            "sort_instructions",
            "sort_instructions_text",
            "sorting_key",
            "stable",
            "stage_description",
            "topic_name",
            "unique",
            "value_serializer",
        }
        required = {
            "current_output_link_type",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "kafka_server_host_name",
            "output_acp_should_hide",
            "topic_name",
        }
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_parameters_props(self) -> dict:
        include, exclude = self.validate_parameters()
        props = {"execution_mode", "input_count", "output_count", "preserve_partitioning"}
        required = set()
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_app_data_props(self) -> dict:
        return {
            "datastage": {
                "active": 1,
                "SupportsRef": True,
                "maxRejectOutputs": 0,
                "minRejectOutputs": 0,
                "maxReferenceInputs": 0,
                "minReferenceInputs": 0,
            }
        }

    def get_input_ports_props(self) -> dict:
        include, exclude = self.validate_target()
        props = {"runtime_column_propagation"}
        required = set()
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property: {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties: {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_input_cardinality(self) -> dict:
        return {"min": 0, "max": 1}

    def get_output_ports_props(self) -> dict:
        include, exclude = self.validate_source()
        props = set()
        required = set()
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property: {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties: {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_output_cardinality(self) -> dict:
        return {"min": 0, "max": 1}

    def get_allowed_as_source_props(self) -> bool:
        return True

    def get_allowed_as_target_props(self) -> bool:
        return True



from ibm_watsonx_data_integration.services.datastage.models.enums import AZURESYNAPSE_CONNECTION
from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestAzureSynapseConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "azuresynapse_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _test_table_1_ = fc.add_stage("Microsoft Azure Synapse Analytics", "test_table_1")
        _test_table_1_.configuration.schema_name = "tm_ds"
        _test_table_1_.configuration.table_name = "test_table"
        _test_table_1_.configuration.connection.authentication_method = AZURESYNAPSE_CONNECTION.AuthMethod.entra_id_user
        _test_table_1_.configuration.connection.database = "CONOPSDB"
        _test_table_1_.configuration.connection.hostname_or_ip_address = environ["AZURESYNAPSE_HOST"]
        _test_table_1_.configuration.connection.password = environ["AZURESYNAPSE_PASSWORD"]
        _test_table_1_.configuration.connection.port = "1433"
        _test_table_1_.configuration.connection.proxy = False
        _test_table_1_.configuration.connection.port_is_ssl_enabled = True
        _test_table_1_.configuration.connection.username = environ["AZURESYNAPSE_USERNAME"]

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        test_table_1_schema = _test_table_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        test_table_1_schema.add_field("BIGINT", "TYPE_BIGINT")
        test_table_1_schema.add_field("BINARY", "TYPE_BINARY").length(10)
        test_table_1_schema.add_field("BIT", "TYPE_BIT")
        test_table_1_schema.add_field("CHAR", "TYPE_CHAR").length(20)
        test_table_1_schema.add_field("DATE", "TYPE_DATE")
        test_table_1_schema.add_field("DECIMAL", "TYPE_DECIMAL").length(5).precision(5).scale(2)
        test_table_1_schema.add_field("FLOAT", "TYPE_DOUBLE")
        test_table_1_schema.add_field("FLOAT", "TYPE_FLOAT")
        test_table_1_schema.add_field("INTEGER", "TYPE_INTEGER")
        test_table_1_schema.add_field("VARCHAR", "TYPE_LONGNVARCHAR").length(4000)
        test_table_1_schema.add_field("VARBINARY", "TYPE_LONGVARBINARY").length(8000)
        test_table_1_schema.add_field("VARCHAR", "TYPE_LONGVARCHAR").length(8000)
        test_table_1_schema.add_field("CHAR", "TYPE_NCHAR").length(10)
        test_table_1_schema.add_field("NUMERIC", "TYPE_NUMERIC").length(5)
        test_table_1_schema.add_field("VARCHAR", "TYPE_NVARCHAR").length(50)
        test_table_1_schema.add_field("REAL", "TYPE_REAL").length(7)
        test_table_1_schema.add_field("SMALLINT", "TYPE_SMALLINT")
        test_table_1_schema.add_field("TIMESTAMP", "TYPE_TIME")
        test_table_1_schema.add_field("TIMESTAMP", "TYPE_TIMESTAMP")
        test_table_1_schema.add_field("TINYINT", "TYPE_TINYINT").unsigned()
        test_table_1_schema.add_field("VARBINARY", "TYPE_VARBINARY").length(10)
        test_table_1_schema.add_field("VARCHAR", "TYPE_VARCHAR").length(50)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import DREMIO_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class AuthType(Enum):
    token = "token"
    user_pass = "user_pass"


class DremioConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "cca8fcca-71f9-4e4f-a8fb-89fd2c0072cf"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    authentication_type: DREMIO_CONNECTION.AuthType | None = Field(
        DREMIO_CONNECTION.AuthType.user_pass, alias="auth_type"
    )
    discover_data_assets: bool | None = Field(None, alias="auto_discovery")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    hostname_or_ip_address: str = Field(None, alias="host")
    password: str = Field(None, alias="password")
    personal_access_token: str = Field(None, alias="pat")
    port: int = Field(None, alias="port")
    dremio_cloud_project_id: str | None = Field(None, alias="project_id")
    port_is_ssl_enabled: bool | None = Field(False, alias="ssl")
    ssl_certificate: str | None = Field(None, alias="ssl_certificate")
    username: str = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    additional_properties: str | None = Field(None, alias="properties")
    ssl_certificate_file: str | None = Field(None, alias="ssl_certificate_file")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        include.add("ssl_certificate") if (self.port_is_ssl_enabled) else exclude.add("ssl_certificate")
        (
            include.add("password")
            if (
                ((not self.defer_credentials))
                and (
                    self.authentication_type
                    and (
                        (hasattr(self.authentication_type, "value") and self.authentication_type.value == "user_pass")
                        or (self.authentication_type == "user_pass")
                    )
                )
            )
            else exclude.add("password")
        )
        (
            include.add("personal_access_token")
            if (
                self.authentication_type
                and (
                    (hasattr(self.authentication_type, "value") and self.authentication_type.value == "token")
                    or (self.authentication_type == "token")
                )
            )
            else exclude.add("personal_access_token")
        )
        (
            include.add("username")
            if (
                ((not self.defer_credentials))
                and (
                    self.authentication_type
                    and (
                        (hasattr(self.authentication_type, "value") and self.authentication_type.value == "user_pass")
                        or (self.authentication_type == "user_pass")
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")
        include.add("additional_properties") if (self.hidden_dummy_property1) else exclude.add("additional_properties")
        include.add("ssl_certificate_file") if (self.hidden_dummy_property1) else exclude.add("ssl_certificate_file")

        (
            include.add("personal_access_token")
            if (
                self.authentication_type
                and (
                    (hasattr(self.authentication_type, "value") and self.authentication_type.value == "token")
                    or (self.authentication_type == "token")
                )
            )
            else exclude.add("personal_access_token")
        )
        (
            include.add("password")
            if (not self.defer_credentials)
            and (
                self.authentication_type
                and (
                    (hasattr(self.authentication_type, "value") and self.authentication_type.value == "user_pass")
                    or (self.authentication_type == "user_pass")
                )
            )
            else exclude.add("password")
        )
        (
            include.add("username")
            if (not self.defer_credentials)
            and (
                self.authentication_type
                and (
                    (hasattr(self.authentication_type, "value") and self.authentication_type.value == "user_pass")
                    or (self.authentication_type == "user_pass")
                )
            )
            else exclude.add("username")
        )
        (
            include.add("ssl_certificate")
            if (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



from ibm_watsonx_data_integration.services.datastage.models.enums import AMAZONS3
from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestAmazonS3Connection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "amazons3_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _fastpathemp2csv0_1_ = fc.add_stage("Amazon S3", "FastpathEmp2csv0_1")
        _fastpathemp2csv0_1_.configuration.ds_file_format = AMAZONS3.DSFileFormat.csv
        _fastpathemp2csv0_1_.configuration.ds_first_line_header = True
        _fastpathemp2csv0_1_.configuration.file_name = "FastpathEmp2.csv.0"

        _fastpathemp2csv0_1_.configuration.connection.access_key = environ["AMAZONS3_ACCESS_KEY"]
        _fastpathemp2csv0_1_.configuration.connection.bucket = "fastpath-bucket"
        _fastpathemp2csv0_1_.configuration.connection.proxy = False
        _fastpathemp2csv0_1_.configuration.connection.secret_key = environ["AMAZONS3_SECRET_KEY"]
        _fastpathemp2csv0_1_.configuration.connection.url = "s3.us-east-1.amazonaws.com"
        _fastpathemp2csv0_1_.configuration.connection.authentication_method = "basic_credentials"

        _peek_1_ = fc.add_stage("Peek", "Peek_1")
        _peek_1_.configuration.runtime_column_propagation = 0

        # Graph
        fastpathemp2csv0_1_schema = _fastpathemp2csv0_1_.connect_output_to(_peek_1_).set_name("Link_4").create_schema()

        # Schemas
        fastpathemp2csv0_1_schema.add_field("VARCHAR", "ggg").nullable().length(1024)
        fastpathemp2csv0_1_schema.add_field("VARCHAR", "ggg__DUPLICATE__1").nullable().length(1024)
        fastpathemp2csv0_1_schema.add_field("VARCHAR", "ggg__DUPLICATE__2").nullable().length(1024)
        fastpathemp2csv0_1_schema.add_field("VARCHAR", "ggg__DUPLICATE__3").nullable().length(1024)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025

"""Module containing Project Models."""

import json
import requests
import warnings
from ibm_watsonx_data_integration.common.models import BaseModel, CollectionModel, CollectionModelResults
from ibm_watsonx_data_integration.common.utils import get_params_from_swagger
from ibm_watsonx_data_integration.cpd_models import (
    Connection,
    Connections,
    DatasourceType,
    Job,
    Jobs,
)
from ibm_watsonx_data_integration.cpd_models.flow_model import Flow
from ibm_watsonx_data_integration.cpd_models.flows_model import Flows
from ibm_watsonx_data_integration.services.streamsets.models import (
    Engine,
    Environment,
    Environments,
    StreamsetsConnection,
    StreamsetsFlow,
)
from ibm_watsonx_data_integration.services.streamsets.models.flow_model import FlowValidationError
from pydantic import ConfigDict, Field, PrivateAttr
from typing import TYPE_CHECKING, Any, ClassVar, Optional

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.platform import Platform


class ProjectMetadata(BaseModel):
    """Model for metadata in a project."""

    guid: str = Field(repr=True)
    url: str = Field(repr=False)
    created_at: str | None = Field(repr=False)
    updated_at: str | None = Field(repr=False)

    model_config = ConfigDict(frozen=True)
    _expose: bool = PrivateAttr(default=False)


class Storage(BaseModel):
    """Model for Storage details in a project."""

    type: str
    guid: str = Field(frozen=True)
    properties: dict[str, Any]
    _expose: bool = PrivateAttr(default=False)


class Scope(BaseModel):
    """Model for Scope details in a project."""

    bss_account_id: str = Field(frozen=True)
    _expose: bool = PrivateAttr(default=False)


class Project(BaseModel):
    """The Model for Projects."""

    metadata: ProjectMetadata = Field(repr=True)

    name: str = Field(repr=True)
    description: str = Field(repr=False)
    type: str | None = Field(repr=False)
    generator: str | None = Field(repr=False)
    public: bool | None = Field(repr=False)
    creator: str | None = Field(repr=False)
    tags: list | None = Field(default_factory=list, repr=False)

    storage: Storage | None = Field(repr=False)
    scope: Scope | None = Field(repr=False)

    EXPOSED_DATA_PATH: ClassVar[dict] = {"entity": {}}

    def __init__(self, platform: Optional["Platform"] = None, **project_json: dict) -> None:
        """The __init__ of the Project class.

        Args:
            platform: The Platform object.
            project_json: The JSON for the Service.
        """
        super().__init__(**project_json)
        self._platform = platform
        self._inital_tags = [] if not hasattr(self, "tags") else list(self.tags)

    def _update_tags(self) -> None:
        """Updates tags of the Project."""
        initial_tags = set(self._inital_tags)
        current_tags = set(self.tags)

        body = []

        tags_to_delete = initial_tags - current_tags
        if tags_to_delete:
            body.append({"op": "remove", "tags": list(tags_to_delete)})

        tags_to_add = current_tags - initial_tags
        if tags_to_add:
            body.append({"op": "add", "tags": list(tags_to_add)})

        if body:
            self._inital_tags = self.tags
            self._platform._project_api.update_tags(self.metadata.guid, json.dumps(body))

    @staticmethod
    def _create(
        platform: "Platform",
        name: str,
        description: str = "",
        tags: list = None,
        public: bool = False,
        project_type: str = "wx",
    ) -> "Project":
        cloud_storage = platform._get_cloud_storage()
        cloud_storage_guid = cloud_storage["guid"]
        cloud_storage_id = cloud_storage["id"]

        data = {
            "name": name,
            "description": description,
            "generator": "watsonx-di-sdk",
            "public": public,
            "tags": ["sdk-tags"] if not tags else tags,
            "storage": {
                "type": "bmcos_object_storage",
                "guid": cloud_storage_guid,
                "resource_crn": cloud_storage_id,
            },
            "type": project_type,
        }

        response = platform._project_api.create_project(json.dumps(data))
        location = response.json()["location"]
        project_id = location.split("/")[-1]

        project_json = platform._project_api.get_project(project_id).json()
        return Project(platform=platform, **project_json)

    def _update(self) -> requests.Response:
        # Update tags
        self._update_tags()

        # Update rest of project
        data = {"name": self.name, "description": self.description, "public": self.public}

        project_json = self.model_dump()
        if "catalog" in project_json:
            data["catalog"] = {"guid": project_json["catalog"]["guid"], "public": project_json["catalog"]["public"]}

        data = json.dumps(data)
        return self._platform._project_api.update_project(id=self.metadata.guid, data=data)

    def _delete(self) -> requests.Response:
        return self._platform._project_api.delete_project(self.metadata.guid)

    @property
    def jobs(self) -> Jobs:
        """Retrieves jobs associated with the project.

        Returns:
            A list of Jobs within the project.
        """
        return Jobs(platform=self._platform, project=self)

    def create_job(
        self,
        name: str,
        flow: Flow,
        configuration: dict[str, Any] | None = None,
        description: str | None = None,
        job_parameters: dict[str, Any] | None = None,
        retention_policy: dict[str, int] | None = None,
        parameter_sets: list[dict[str, str]] | None = None,
        schedule: str | None = None,
        schedule_info: dict[str, Any] | None = None,
    ) -> Job:
        """Create Job for given asset.

        Args:
            name: Name for a Job.
            flow: A reference to the flow for which the job will be created.
            configuration: Environment variables for a Job.
            description: Job description.
            job_parameters: Parameters use internally by a Job.
            retention_policy: Retention policy for a Job.
            parameter_sets: Parameter sets for a Job.
            schedule: Crone string.
            schedule_info: Schedule info for a Job.

        Returns:
            A Job instance.

        Raises:
            TypeError: If both asset_ref and asset_ref_type are provided, or if neither is provided
        """
        return Job._create(
            self,
            name,
            flow,
            configuration,
            description,
            job_parameters,
            retention_policy,
            parameter_sets,
            schedule,
            schedule_info,
        )

    def delete_job(self, job: Job) -> requests.Response:
        """Allows to delete specified Job within project.

        Args:
            job: Instance of a Job to delete.

        Returns:
            A HTTP response. If it is 204, then the operation completed successfully.
        """
        return job._delete()

    def update_job(self, job: Job) -> requests.Response:
        """Allows to update specified Job within a project.

        Args:
            job: Instance of a Job to update.

        Returns:
            A HTTP response. If it is 200, then the operation completed successfully.
        """
        return job._update()

    @property
    def environments(self) -> Environments:
        """Retrieves environments associated with the project.

        Returns:
            A list of Environments within the project.
        """
        return Environments(platform=self._platform, project=self)

    def create_environment(
        self,
        *,
        name: str,
        engine_version: str | None = None,
        description: str = None,
        engine_type: str = "data_collector",
        engine_properties: dict = None,
        log4j2_properties: dict = None,
        external_resource_asset: dict = None,
        stage_libs: list = None,
        jvm_options: list = None,
        max_cpu_load: float = None,
        max_memory_used: float = None,
        max_jobs_running: int = None,
        engine_heartbeat_interval: int = None,
        cpus_to_allocate: float = None,
    ) -> Environment:
        """Allows to create a new Environment within project.

        All of not set parameters will be skipped and set with default values provided by backed.

        Args:
            name: Name of the environment.
            description: Description of the environment.
            engine_type: Type of the engine.
            engine_version: Version of the engine. Default is the latest engine version.
            engine_properties: Properties of the engine.
            external_resource_asset: External resources.
            log4j2_properties: Log4j2 properties.
            stage_libs: Stage libraries.
            jvm_options: JVM options.
            max_cpu_load: Maximum CPU load.
            max_memory_used: Maximum memory used.
            max_jobs_running: Maximum jobs running.
            engine_heartbeat_interval: Engine heartbeat interval.
            cpus_to_allocate: Number of CPU used.

        Returns:
            The created environment.
        """
        return Environment._create(
            self,
            name,
            engine_version,
            description,
            engine_type,
            engine_properties,
            log4j2_properties,
            external_resource_asset,
            stage_libs,
            jvm_options,
            max_cpu_load,
            max_memory_used,
            max_jobs_running,
            engine_heartbeat_interval,
            cpus_to_allocate,
        )

    def delete_environment(self, environment: Environment) -> requests.Response:
        """Allows to delete specified Environment within a Project.

        Args:
            environment: Instance of an Environment to delete.

        Returns:
            A HTTP response.
        """
        return environment._delete()

    def update_environment(self, environment: Environment) -> requests.Response:
        """Allows to update specified Environment within a Project.

        Args:
            environment: Instance of an Environment to update.

        Returns:
            A HTTP response.
        """
        return environment._update()

    @property
    def flows(self) -> Flows:
        """Returns Flows from the Project."""
        return Flows(project=self)

    def delete_flow(self, flow: Flow) -> requests.Response:
        """Delete a Flow.

        Args:
            flow: The Flow object.

        Returns:
            A HTTP response.
        """
        return flow._delete()

    def create_flow(
        self, name: str, environment: Environment, description: str = "", flow_type: str = "streamsets"
    ) -> Flow:
        """Creates a Flow.

        Args:
            name: The name of the flow.
            environment: The environment which will be used to run this flow.
            description: The description of the flow.
            flow_type: The type of flow (must be registered in Flow.flow_registry).

        Returns:
            The created Flow subclass instance (StreamsetsFlow by default).

        """
        try:
            flow = Flow._flow_registry[flow_type]
        except ValueError:
            raise TypeError(f"Flow type '{flow_type}' is not supported. Available: {list(Flow._flow_registry)}")

        flow = flow._create(
            project=self, name=name, environment=environment, description=description, flow_type=flow_type
        )
        return flow

    def update_flow(self, flow: Flow) -> requests.Response:
        """Update a Flow.

        Args:
            flow: The Flow object.

        Returns:
            A HTTP response.
        """
        return flow._update()

    def duplicate_flow(self, flow: Flow, name: str, description: str = "") -> Flow:
        """Duplicate a Flow.

        Args:
            flow: The Flow.
            name: The name of the flow.
            description: The description of the flow.

        Returns:
            A copy of passed flow.
        """
        return flow._duplicate(name, description)

    def validate_flow(self, flow: StreamsetsFlow) -> list[FlowValidationError]:
        """Validates a flow.

        Args:
            flow: The Flow to validate.

        Returns:
            A `list` of `FlowValidationError` containing issues.
        """
        warnings.warn(
            "Project.validate_flow() is now deprecated. Use StreamsetsFlow.validate() instead.", DeprecationWarning
        )
        return flow.validate()

    @property
    def engines(self) -> list[Engine]:
        """Returns a list of Engines within project."""
        # TODO: replace with CollectionModel when implemented
        query_params = {
            "project_id": self.metadata.guid,
        }
        api_response = self._platform._engine_api.get_engines(params=query_params).json()

        return [
            Engine(platform=self._platform, project=self, **engine_json)
            for engine_json in api_response.get("streamsets_engines", [])
        ]

    def get_engine(self, engine_id: str) -> Engine:
        """Retrieve an engine by its engine_id.

        Args:
            engine_id (str): The asset_id of the engine to retrieve.

        Returns:
            Engine: The retrieved engine.

        Raises:
            HTTPError: If the request fails.
        """
        query_params = {
            "project_id": self.metadata.guid,
        }
        api_response = self._platform._engine_api.get_engine(engine_id=engine_id, params=query_params)
        return Engine(platform=self._platform, project=self, **api_response.json())

    def delete_engine(self, engine: Engine) -> requests.Response:
        """Allows to delete specified Engine within project.

        Args:
            engine: Instance of an Engine to delete.

        Returns:
            A HTTP response.
        """
        query_params = {"project_id": self.metadata.guid}
        return self._platform._engine_api.delete_engine(engine_id=engine.engine_id, params=query_params)

    @property
    def connections(self) -> Connections:
        """Retrieves connections associated with the project.

        Returns:
            A Connections object.
        """
        return Connections(platform=self._platform, project=self)

    def create_connection(
        self,
        name: str,
        datasource_type: DatasourceType,
        description: str | None = None,
        properties: dict | None = None,
        test: bool = True,
    ) -> Connection:
        """Create a Connection.

        Args:
            name: name for the new connection.
            description: description for the new connection.
            datasource_type: type of the datasource.
            properties: properties of the new connection.
            test: whether to test the connection before saving it.
                  If true and validation cannot be estabilished, connection will not be saved.

        Returns:
            Created Connection object.
        """
        return Connection._create(self, name, datasource_type, description, properties, test)

    def delete_connection(self, connection: Connection) -> requests.Response:
        """Remove the Connection.

        Args:
            connection: connection to delete

        Returns:
            A HTTP response.
        """
        return connection._delete()

    def update_connection(self, connection: Connection, test: bool = True) -> requests.Response:
        """Update the Connection.

        Args:
            connection: connection to update
            test: whether to test the connection before saving it.
                  If true and validation cannot be estabilished, connection will not be saved.

        Returns:
            A HTTP response.
        """
        return connection._update(test=test)

    def copy_connection(self, connection: Connection) -> Connection:
        """Update the Connection.

        Args:
            connection: connection to copy

        Returns:
            Copied Connection object.
        """
        return connection._copy()

    @property
    def _streamsets_connections(self) -> list[StreamsetsConnection]:
        """Retrieves StreamSets connections associated with the project.

        Returns:
            A list of StreamsetsConnection within the project.
        """
        connections_json = self._platform._streamsets_flow_api.get_streamsets_connections(
            params={
                "project_id": self.metadata.guid,
            },
        ).json()

        return [
            StreamsetsConnection(
                platform=self._platform,
                project=self,
                **{
                    "entity": {
                        "datasource_type": connection["datasource_type"],
                        "name": connection["name"],
                    },
                    "metadata": {
                        "asset_id": connection["asset_id"],
                    },
                },
            )
            for connection in connections_json.get("connections", list())
        ]

    def _get_streamsets_connection(
        self, connection_id: str, version: str | None = None, type: str | None = None
    ) -> StreamsetsConnection:
        """Retrieves StreamSets connection by id associated with the project.

        Args:
            connection_id: Id of the connection asset to retrieve.
            version: Connection version.
            type: Connection type.

        Returns:
            Retrieved StreamsetsConnection object.
        """
        params = {
            "project_id": self.metadata.guid,
        }
        if version:
            params["connection_version"] = version
        if type:
            params["connection_type"] = type

        connection_json = self._platform._streamsets_flow_api.get_streamsets_connection(
            connection_id=connection_id,
            params=params,
        ).json()

        # NOTE: connection_json has also 'alternative_mapping' reserved for future use
        #       this object is stripped for now as it is filled with nulls
        return StreamsetsConnection(platform=self._platform, project=self, **connection_json["connection"])


class Projects(CollectionModel):
    """Collection of Project instances."""

    def __init__(self, platform: "Platform") -> None:
        """The __init__ of the Projects class.

        Args:
            platform: The Platform object.
        """
        super().__init__(platform)
        self.unique_id = "metadata.guid"

    def __len__(self) -> int:
        """Total amount of projects."""
        return self._platform._project_api.get_projects_total().json()["total"]

    def _request_parameters(self) -> list:
        request_params = ["project_id"]
        content_string = self._platform._project_api.get_swagger().content.decode("utf-8")
        request_path = f"/{self._platform._project_api.url_path}"
        request_params.extend(get_params_from_swagger(content_string=content_string, request_path=request_path))
        return request_params

    def _get_results_from_api(self, request_params: dict = None, **kwargs: dict) -> CollectionModelResults:
        """Returns results of an api request."""
        request_params_defaults = {
            "project_id": None,
            "bss_acount_id": None,
            "type": None,
            "member": None,
            "roles": None,
            "tag_names": None,
            "name": None,
            "match": None,
            "project_ids": None,
            "include": "name,fields,members,tags,settings",
            "limit": 100,
            "bookmark": None,
        }
        request_params_unioned = request_params_defaults
        request_params_unioned.update(request_params)
        project_id = request_params_unioned.get("project_id")

        if project_id:
            response = self._platform._project_api.get_project(project_id).json()
            response = {"resources": [response]}
        else:
            response = self._platform._project_api.get_projects(
                params={k: v for k, v in request_params_unioned.items() if v is not None}
            ).json()
        return CollectionModelResults(
            response,
            Project,
            "bookmark",
            "bookmark",
            "resources",
            {"platform": self._platform},
        )



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import JDBC_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class RowLimitSupport(Enum):
    none = "none"
    prefix = "prefix"
    suffix = "suffix"


class JdbcConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "e59b1c36-6f30-4879-9f74-7e81dde4cca6"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    allow_filter_pushdown: bool | None = Field(None, alias="allow_push_filters")
    batch_size: int | None = Field(None, alias="batch_size")
    case_sensitive_ids: bool | None = Field(None, alias="case_sensitive_ids")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    fetch_size: int | None = Field(None, alias="fetch_size")
    gateway_url: str | None = Field(None, alias="gateway_url")
    jdbc_driver_files: str | None = Field(None, alias="jar_uris")
    jdbc_driver_class: str = Field(None, alias="jdbc_driver")
    jdbc_properties: str | None = Field(None, alias="jdbc_properties")
    jdbc_url: str = Field(None, alias="jdbc_url")
    password: str | None = Field(None, alias="password")
    row_limit_prefix: str | None = Field(None, alias="row_limit_prefix")
    row_limit_suffix: str | None = Field(None, alias="row_limit_suffix")
    row_limit_support: JDBC_CONNECTION.RowLimitSupport | None = Field(None, alias="row_limit_support")
    secure_gateway_id: str | None = Field(None, alias="sg_gateway_id")
    sg_host_original: str | None = Field(None, alias="sg_host_original")
    secure_gateway_as_http_proxy: bool | None = Field(None, alias="sg_http_proxy")
    secure_gateway_security_token: str | None = Field(None, alias="sg_security_token")
    secure_gateway_service_url: str | None = Field(None, alias="sg_service_url")
    port_is_ssl_enabled: bool | None = Field(False, alias="ssl")
    ssl_certificate: str | None = Field(None, alias="ssl_certificate")
    table_types: str | None = Field(None, alias="table_types")
    trust_all_ssl_certificates: bool | None = Field(False, alias="trust_all_ssl_cert")
    username: str | None = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    auto_commit_in_metadata_discovery: bool | None = Field(None, alias="auto_commit_discovery")
    connector_uses_catalog_structure: bool | None = Field(False, alias="catalog_support")
    satellite_client_certificate: str | None = Field(None, alias="sl_client_cert")
    satellite_client_private_key: str | None = Field(None, alias="sl_client_private_key")
    satellite_connector_id: str | None = Field(None, alias="sl_connector_id")
    satellite_endpoint_host: str | None = Field(None, alias="sl_endpoint_host")
    satellite_endpoint_display_name: str | None = Field(None, alias="sl_endpoint_name")
    satellite_endpoint_port: int | None = Field(None, alias="sl_endpoint_port")
    sl_host_original: str | None = Field(None, alias="sl_host_original")
    satellite_as_http_proxy: bool | None = Field(None, alias="sl_http_proxy")
    satellite_location_id: str | None = Field(None, alias="sl_location_id")
    satellite_service_url: str | None = Field(None, alias="sl_service_url")
    use_column_name_in_the_statements: bool | None = Field(None, alias="use_column_name")
    ssl_certificate_file: str | None = Field(None, alias="ssl_certificate_file")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        (
            include.add("ssl_certificate")
            if ((((not self.ssl_certificate_file))) and (self.port_is_ssl_enabled))
            else exclude.add("ssl_certificate")
        )
        (
            include.add("row_limit_suffix")
            if (
                self.row_limit_support
                and (
                    (hasattr(self.row_limit_support, "value") and self.row_limit_support.value == "suffix")
                    or (self.row_limit_support == "suffix")
                )
            )
            else exclude.add("row_limit_suffix")
        )
        (
            include.add("row_limit_prefix")
            if (
                self.row_limit_support
                and (
                    (hasattr(self.row_limit_support, "value") and self.row_limit_support.value == "prefix")
                    or (self.row_limit_support == "prefix")
                )
            )
            else exclude.add("row_limit_prefix")
        )
        (
            include.add("satellite_connector_id")
            if (((not self.secure_gateway_id)) and (not self.satellite_location_id))
            else exclude.add("satellite_connector_id")
        )
        (
            include.add("satellite_location_id")
            if (((not self.secure_gateway_id)) and (not self.satellite_connector_id))
            else exclude.add("satellite_location_id")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        (
            include.add("satellite_client_certificate")
            if (self.hidden_dummy_property1)
            else exclude.add("satellite_client_certificate")
        )
        (
            include.add("secure_gateway_service_url")
            if (self.hidden_dummy_property1)
            else exclude.add("secure_gateway_service_url")
        )
        include.add("sl_host_original") if (self.hidden_dummy_property1) else exclude.add("sl_host_original")
        (
            include.add("satellite_client_private_key")
            if (self.hidden_dummy_property1)
            else exclude.add("satellite_client_private_key")
        )
        (
            include.add("satellite_connector_id")
            if (self.hidden_dummy_property1)
            else exclude.add("satellite_connector_id")
        )
        include.add("sg_host_original") if (self.hidden_dummy_property1) else exclude.add("sg_host_original")
        (
            include.add("secure_gateway_as_http_proxy")
            if (self.hidden_dummy_property1)
            else exclude.add("secure_gateway_as_http_proxy")
        )
        include.add("secure_gateway_id") if (self.hidden_dummy_property1) else exclude.add("secure_gateway_id")
        (
            include.add("secure_gateway_security_token")
            if (self.hidden_dummy_property1)
            else exclude.add("secure_gateway_security_token")
        )
        include.add("satellite_service_url") if (self.hidden_dummy_property1) else exclude.add("satellite_service_url")
        (
            include.add("satellite_endpoint_display_name")
            if (self.hidden_dummy_property1)
            else exclude.add("satellite_endpoint_display_name")
        )
        (
            include.add("satellite_endpoint_host")
            if (self.hidden_dummy_property1)
            else exclude.add("satellite_endpoint_host")
        )
        (
            include.add("satellite_endpoint_port")
            if (self.hidden_dummy_property1)
            else exclude.add("satellite_endpoint_port")
        )
        include.add("ssl_certificate_file") if (self.hidden_dummy_property1) else exclude.add("ssl_certificate_file")
        (
            include.add("satellite_as_http_proxy")
            if (self.hidden_dummy_property1)
            else exclude.add("satellite_as_http_proxy")
        )
        include.add("satellite_location_id") if (self.hidden_dummy_property1) else exclude.add("satellite_location_id")

        (
            include.add("row_limit_prefix")
            if (
                self.row_limit_support
                and (
                    (
                        hasattr(self.row_limit_support, "value")
                        and self.row_limit_support.value
                        and "prefix" in str(self.row_limit_support.value)
                    )
                    or ("prefix" in str(self.row_limit_support))
                )
            )
            else exclude.add("row_limit_prefix")
        )
        (
            include.add("satellite_connector_id")
            if (not self.secure_gateway_id) and (not self.satellite_location_id)
            else exclude.add("satellite_connector_id")
        )
        (
            include.add("row_limit_suffix")
            if (
                self.row_limit_support
                and (
                    (
                        hasattr(self.row_limit_support, "value")
                        and self.row_limit_support.value
                        and "suffix" in str(self.row_limit_support.value)
                    )
                    or ("suffix" in str(self.row_limit_support))
                )
            )
            else exclude.add("row_limit_suffix")
        )
        (
            include.add("ssl_certificate_file")
            if (not self.ssl_certificate) and (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate_file")
        )
        (
            include.add("ssl_certificate")
            if (not self.ssl_certificate_file) and (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate")
        )
        (
            include.add("satellite_location_id")
            if (not self.secure_gateway_id) and (not self.satellite_connector_id)
            else exclude.add("satellite_location_id")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



from enum import Enum
from pydantic import BaseModel, Field


class BufMode(Enum):
    default = "default"
    auto_buffer = "autobuffer"
    buffer = "buffer"
    no_buffer = "nobuffer"


class BaseStage(BaseModel):
    buf_mode: BufMode | str = Field(BufMode.default, alias="buf_mode")
    max_mem_buf_size: int = Field(3145728, alias="max_mem_buf_size")
    buf_free_run: int = Field(50, alias="buf_free_run")
    queue_upper_size: int = Field(0, alias="queue_upper_size")
    disk_write_inc: int = Field(1048576, alias="disk_write_inc")

    def validate(self):
        include = set()
        exclude = set()

        (
            include.add("max_mem_buf_size")
            if (
                ((hasattr(self.buf_mode, "value") and self.buf_mode.value != "default") or self.buf_mode != "default")
                and (
                    (hasattr(self.buf_mode, "value") and self.buf_mode.value != "no_buffer")
                    or self.buf_mode.value != "nobuffer"
                )
            )
            else exclude.add("max_mem_buf_size")
        )
        (
            include.add("buf_free_run")
            if (
                ((hasattr(self.buf_mode, "value") and self.buf_mode.value != "default") or self.buf_mode != "default")
                and (
                    (hasattr(self.buf_mode, "value") and self.buf_mode.value != "no_buffer")
                    or self.buf_mode.value != "nobuffer"
                )
            )
            else exclude.add("buf_free_run")
        )
        (
            include.add("queue_upper_size")
            if (
                ((hasattr(self.buf_mode, "value") and self.buf_mode.value != "default") or self.buf_mode != "default")
                and (
                    (hasattr(self.buf_mode, "value") and self.buf_mode.value != "no_buffer")
                    or self.buf_mode.value != "nobuffer"
                )
            )
            else exclude.add("queue_upper_size")
        )
        (
            include.add("disk_write_inc")
            if (
                ((hasattr(self.buf_mode, "value") and self.buf_mode.value != "default") or self.buf_mode != "default")
                and (
                    (hasattr(self.buf_mode, "value") and self.buf_mode.value != "no_buffer")
                    or self.buf_mode.value != "nobuffer"
                )
            )
            else exclude.add("disk_write_inc")
        )

        return include, exclude

    def get_advanced_props(self):
        include, exclude = self.validate()
        return self.model_dump(include=include, exclude=exclude, by_alias=True, exclude_none=True)
